#!/bin/bash
################################################################################
#
# Course Name:  Install MiniSAP on Your Personal Couputer
# Course URI:   https://www.installminisap.com/
# Script Owner: Any user in Virtual Machine guest operating system (OpenSUSE 15.1 Leap).
# Script Home:  /media/sf_MiniSAP/GuestOS/MiniSAP-Plus-0.4.3/install.sh
# Description:  This is MiniSAP+, an everyday utility for limited access
#               user accounts to start/stop MiniSAP services, without having
#               to continually login as the '<sid>adm' user.
# Valid For:    SAP Netweaver AS ABAP 7.50 SP02 Developer Edition (or above) for Linux.
# Author Name:  John Lang
# Author URI:   https://www.johnlang.com.au/
#
# - Why? Creates a 'MiniSAP+' gnome aplication with popup menu to easily
#        start/stop MiniSAP services.
#
# - Installation Instructions - Execute this script as the root user:
#   1. Open a new terminal window in the Virtual Machine guest operating system (OpenSUSE 15.1 Leap).
#   2. Type: sudo /media/sf_MiniSAP/GuestOS/MiniSAP-Plus-0.4.3/install.sh
#   3. Enter the administrator password for root access, then press [Enter].
#   4. Click the [Activities] system menu.
#   5. Type 'minisap' in the application search bar.
#      Tip: Reboot the guest operating system (OpenSUSE 15.1 Leap) if the 'MiniSAP+' application is not found.
#   6. Right-click the 'MiniSAP+' icon, click 'Add to Favourites' popup menu item.
#   7. Click the [Activities] system menu again (twice) to see 'MiniSAP+' in the left side bar.
#   8. Drag the 'MiniSAP+' favourite to the top of the list.
#   9. Right-click the 'MiniSAP+' favourite application to see available actions in popup menu.
#
# - OpenSUSE 15.1 Leap has '/usr/local/bin/' in the execution search $PATH variable for normal users.
# - Target Directory and File Map:
#   /opt/minisap-plus/
#     SymLink -> /opt/dataprofessionals.com.au/minisap-plus-0.4.3/
#     Note: All install.sh executions should point to their directory.
#     Note: So that /opt/minisap-plus/ becomes the "Installed" package.
#     Note: Now .desktop files can use "/opt/minisap-plus/install.sh --uninstall-now".
#   /opt/minisap.dataprofessionals.com.au/minisap-plus-0.4.3/
#   /usr/local/bin/
#     [rwxrwxr-x root users] minisap-plus.sh
#     [rwxrwxr-x root users] minisapstart.sh
#     [rwxrwxr-x root users] minisapstop.sh
#   /usr/local/share/minisap-plus/
#     [rw-r--r-- root root] com.installminisap.MiniSAPPlus.About.html
#     [rw-r--r-- root root] com.installminisap.MiniSAPPlus.About.Icon.png
#     [rw-r--r-- root root] com.installminisap.MiniSAPPlus.opensuse-gnome-menu.png
#     [rw-r--r-- root root] com.installminisap.MiniSAPPlus.opensuse-gnome-menu-abap.png
#   /usr/share/applications/
#     [rw-r--r-- root root] minisap-plus.desktop
#     [rw-r--r-- root root] minisap-plus-abap-stack.desktop
#     [rw-r--r-- root root] minisap-plus-hana-stack.desktop
#   /usr/share/pixmaps/
#     [rw-r--r-- root root] minisap-plus.png
#     [rw-r--r-- root root] minisap-plus-abap-stack.png
#     [rw-r--r-- root root] minisap-plus-hana-stack.png
#
# - Exit return codes for this script:
#   - exit 0;  # 0 is 'Ok'.
#   - exit 1;  # 1 is 'Error - Invalid arguments passed to script.'.
#   - exit 2;  # 2 is 'Error - Expecting root user access for installation.'.
#   - exit 3;  # 3 is 'Error - Internal variable 'mspFileListFQN=()' array needs attention.'.
#   - exit 4;  # 4 is 'Error - Internal variable 'mspFileListFQN=()' array STILL needs attention.'.
#   - exit 5;  # 5 is 'Error - Internal variable 'mspFileListFQN=()' array and 'mspFileListFQNDeleteMeHousekeeping=()' STILL needs attention.'.
#
# Read Me: https://doc.opensuse.org/documentation/leap/startup/html/book.opensuse.startup/cha-adm-shell.html#sec-adm-dirstructure
# Read Me: https://www.pathname.com/fhs/pub/fhs-2.3.html#OPTADDONAPPLICATIONSOFTWAREPACKAGES
# Read Me: https://docs.oracle.com/cd/E88353_01/html/E37839/xdg-desktop-menu-1.html#scrolltoc
# Read Me: https://developer.gnome.org/integration-guide/stable/preface.html.en
# Read Me: https://specifications.freedesktop.org/desktop-entry-spec/latest/index.html
# Read Me: https://developer.gnome.org/integration-guide/stable/desktop-files.html.en
# Read Me: https://specifications.freedesktop.org/menu-spec/latest/apcs03.html
# Read Me: https://people.freedesktop.org/~hughsient/temp/AppData_WhitePaper.pdf
#
# - Disclaimer: This is definetly not the right way to get it installed into
#   OpenSUSE 15.1 Leap for anything officially supported or updated. Short
#   answer is, this is not my expertise and I just wanted it up and running
#   for my personal journey. You can use it, at your own risk because this is
#   an ugly installer, with no proper package file management, etc.
# - If you are following the www.installminisap.com course then it will work
#   for the software combination specifically documented in that course.
#
# - To-Do: Add update check to autostart (with version comparision).
# - To-Do: Add User prompts for entering user name and password.
#          With --quiet & --reset combination to reset to defaults.
# - To-Do: Populate the 'about' page with something basic.
# - To-Do: Populate the support page on www.installminisap.com.
# - To-Do: Populate the manual page on www.installminisap.com.
# - To-Do: Fill in the minisapplus course ...
# - To-Do: What is the TryExec for Hana file? Add to installer and desktop files.
# - One Day: Add an execution log file capability, like most other scripts.
#
# Investigating - Repositories:
# - http://download.opensuse.org/distribution/leap/$releasever/repo/oss
#         [Provider]            [Type]       [Name]           [Method] [License]
#                               /distribution/                /repo/   /oss/
#                               /source      /                /iso /   /non-oss/
#                               /debug       /                /live/
#                               /factory     /                /jeos/
#                               /ports       /
#                               /tunbleweed  /
#                               /repositories/
#
# - What would it look like from http://download.dataprofessionals.com.au/ perspective?
#
# - /distribution/learn/1.0/iso /minisap-plus/
# - /distribution/learn/1.0/repo/minisap-plus/unstable/
# - /distribution/learn/1.0/repo/minisap-plus/alpha/
# - /distribution/learn/1.0/repo/minisap-plus/beta/
# - /distribution/learn/1.0/repo/minisap-plus/
# - /distribution/learn/1.0/repo/non-oss/
#
# - /repositories/learn/1.0/minisap-plus/rpm/noarch  /     # Archicecture independant '*.rpm's.
# - /repositories/learn/1.0/minisap-plus/rpm/repodata/     # The 'repomd.xml' lives here.
# - /repositories/learn/1.0/minisap-plus/rpm/src     /
# - /repositories/learn/1.0/minisap-plus/rpm/x86_64  /     # x86_64 bit architecture '*.rpm's.
# - /repositories/learn/1.0/minisap-plus/rpm/<package-name><-life-cycle-stage>.repo
#         --> OBS Project -->            .../minisap-plus   -unstable         .repo  # For developers and unit testers.
#         --> OBS Project -->            .../minisap-plus   -alpha            .repo  # Prototyping with 'Curious' user community.
#         --> OBS Project -->            .../minisap-plus   -beta             .repo  # Feedback from 'Courageous' user community.
#         --> OBS Project -->            .../minisap-plus                     .repo  # Production release for the user community.
#
# - Explore using ... http://download.dataprofessionals.com.au/
# - /repository/learn/1.0/minisap/script/<package-name><-life-cycle-stage>.conf
#                                    .../minisap-plus   -unstable         .conf  # For developers and unit testers.
#                                    .../minisap-plus   -alpha            .conf  # Prototyping with 'Curious' user community.
#                                    .../minisap-plus   -beta             .conf  # Feedback from 'Courageous' user community.
#                                    .../minisap-plus   -stable           .conf  # Production release for the user community.
# - /distribution/learn/1.0/script/minisap-plus/unstable/minisap-plus-0.4.1.zip
#                                              /alpha   /minisap-plus-0.4.4.zip
#                                              /beta    /minisap-plus-0.4.7.zip
#                                              /stable  /minisap-plus-0.4.9.zip
#
################################################################################
# [START EDITING] Add the user doco functions here.
################################################################################

mspOSMiniSAPAdminPasswordDefault="MiniSAP1"
mspOSMiniSAPAdminUserNameDefault=""
mspOSMiniSAPReleaseLifecycleDefault="stable"

mspFinishWithSleep="180"
mspVersion="0.4.3"
mspProviderName="dataprofessionals.com.au"
mspPackageName="minisap-plus"

# Itemise files to be installed from the /opt/ package directory.
# Do NOT list the 'install.sh' or 'uninstall.sh' files. Leave them in '/opt/'.
# Always specify the full path of each leaf directory to minise processing.
# Always specify a filename prefix when using '*' wildcard ('/*' not allowed).
# Itemise each '.sh' file to enable 'E(x)ecute' mode for that file.
# Itemise each '.desktop' file to enable 'xdg-desktop-menu' execution.
mspFileListFQN=('/etc/minisap-plus/minisap-plus-example.conf'
                '/etc/xdg/autostart/minisap-plus-autostart.desktop'
                '/usr/local/bin/minisap-plus.sh'
                '/usr/local/bin/minisapstart.sh'
                '/usr/local/bin/minisapstop.sh'
                '/usr/local/share/minisap-plus/about/minisap-plus*'
                '/usr/share/applications/minisap-plus.desktop'
                '/usr/share/applications/minisap-plus-abap-stack.desktop'
                '/usr/share/pixmaps/minisap-plus*'
               )
mspFileListFQNDeleteMeHousekeeping=('/etc/minisap-plus/minisap-plus.conf'
                                    '/tmp/minisap*'
                                    '/tmp/minisap-plus/minisap*'
                                    '/var/cache/minisap*'
                                    '/var/lock/minisap*'
                                    '/var/spool/minisap*'
                                    '/var/tmp/minisap*'
                                   )



################################################################################
# [DO NOT EDIT] This next section of code is mostly a cookie template.
################################################################################

#-------------------------------------------------------------------------------
# Custom functions for execution of this scripts capability.

mspScriptFirstSteps() { echo -n "    Step 1 - Open a new terminal window in the virtual machines guest operating
             system, OpenSUSE 15.x (Leap).

             a. Click the [Activities] system menu.
                Tip - It is the Gnome/OpenSUSE desktop in the top left corner.

             b. Type 'terminal' in application search bar, then press [Enter].

    Step 2 - Type the next line in the new terminal window:
             sudo /media/sf_MiniSAP/GuestOS/MiniSAP-Plus-${mspVersion}/install.sh

       Tip - If you are not following the 'Install MiniSAP' course, then change
             the current directory to be 'MiniSAP-Plus-${mspVersion}/' and then type:
             sudo ./install.sh

    Step 3 - Enter administrator password for root access, then press [Enter].
"
}

mspScriptNextSteps() { echo -n "    Step 4 - Click the [Activities] system menu.

       Tip - It is the Gnome/OpenSUSE desktop in the top left corner.

    Step 5 - Type 'minisap' in the application search bar.

       Tip - Reboot the guest operating system (OpenSUSE 15.1 Leap),
             if the 'MiniSAP+' application is not in search results.

    Step 6 - Right-click the 'MiniSAP+' icon;
             Then click 'Add to Favourites' popup menu item.

    Step 7 - Click the [Activities] system menu again (twice, slowly)
             to see 'MiniSAP+' in the left side bar (the favourites).

    Step 8 - Drag the 'MiniSAP+' favourite to the top of the list.

    Step 9 - Right-click the 'MiniSAP+' favourite application
             to see the available actions in the popup menu.

    Tip: Once either of the 'MiniSAP Developer Editions (ABAP/HANA)'
         have been installed in this OpenSUSE virtual machine,
         repeat Steps 4 to Step 9 again to show the technology stack
         applications 'MiniSAP+ (ABAP)' and 'MiniSAP+ (HANA)'. They
         will allow you to easily perform stack specific actions:
         - 'Start MiniSAP Services' from a restricted access user.
         - 'Stop MiniSAP Services' from a restricted access user.

    Tip: Type 'minisap-plus.sh --help' in a terminal window at any time
         to see a more comprehensive list of available features.

"
}

#-------------------------------------------------------------------------------
# Common functions for execution of this scripts core capabilities.

mspScriptUsage() { echo -n "
NAME
    ${mspScriptFilename} - Install 'MiniSAP Plus (+)' for Gnome & OpenSUSE.

SYNOPSIS
    sudo ${mspScriptDirectory}${mspScriptFilename} [OPTION]...

DESCRIPTION
    Install the 'MiniSAP Plus (+)' utilities for Gnome & OpenSUSE 15.x (Leap).

    A collection of quick start links that improve the MiniSAP Administrators
    experience while learning SAP technologies, in OpenSUSE 15.x (Leap).

    Must be installed as 'root' user without any login shell environment
    initialisation, using 'sudo' not 'sudo -i'.

    The '--not-root' option still requires the current user to have root-like
    levels of access but is specifically not the root user. Will still need
    'rwx' access to directories like /opt/, /usr/local/ and /usr/share/.

OPTIONS
        --debug             Execute script in BASH debug mode. (set -x)
        --finish-key        Finish script execution when [Enter] key pressed.
        --finish-time       Finish script execution with timout (sleep ${mspFinishWithSleep}).
    -h, --help              Display this help and exit.
        --not-root          Try execution as current user. (not root user)
    -p, --password=<val>    Use <val> as password. (where required)
    -q, --quiet             Execute quietly. Do not ask user anything.
    -s, --silent            Execute silently. Output nothing.
        --strict            Exit script on null variables. (set -o nounset)
        --uninstall-now     Uninstall 'MiniSAP+' from system. (Leave in /opt/)
        --update-now=<val>  Update 'MiniSAP+' from installation repository.
    -u, --username=<val>    Use <val> as user name. (Like NPLADM & HXEADM)
    -v, --verbose           Output more information. (echo 'verbose')
        --version           Output version information and exit.

    Short options can be combined together. (-qv)

INSTRUCTIONS
    ${mspScriptDirectory}${mspScriptFilename} --help

    ${mspScriptDirectory}${mspScriptFilename} -p <password>
        Use this <password> instead. Ignore hardcoded value.

    ${mspScriptDirectory}${mspScriptFilename} -u <username> -p <password>
        Use this <username> and <password> instead. Ignore hardcoded value.

    ${mspScriptDirectory}${mspScriptFilename} --update-now=<lifecycle>
        '' ........... to use 'Lifecycle' in local configuration file.
        'stable' ..... for production release to general user community.
        'beta' ....... for feedback from the 'Courageous' user community.
        'alpha' ...... for prototyping with the 'Curious' user community.
        'unstable' ... for developers and unit testers.

    ${mspScriptDirectory}${mspScriptFilename} --verbose --finish-time
        Verbose output and wait for '${mspFinishWithSleep}' before exiting.

INSTRUCTIONS FOR MANUAL STEPS
$(mspScriptFirstSteps)

$(mspScriptNextSteps)

EXIT STATUS CODES
    exit 0  Ok.
    exit 1  Error - Invalid arguments passed to script.
    exit 2  Error - Expecting root user access for installation.
    exit 3  Error - Internal 'mspFileListFQN=()' array needs attention.
    exit 4  Error - Internal 'mspFileListFQN=()' array STILL broken.
    exit 5  Error - Internal 'mspFileListFQN=()' array and
                    'mspFileListFQNDeleteMeHousekeeping=()' STILL broken.
    Note: Status 3, 4, 5 mean this install script is not properly configured.

AUTHOR
    Written by John Thomas Lang.

REPORTING BUGS
    Support (Online): <https://www.installminisap.com/minisap-plus/support/>

COPYRIGHT
    (C) 2020 <https://www.installminisap.com/> (R) All rights reserved.
    License: Creative Commons Attribution-NonCommercial-ShareAlike v4.0.
    Online: CC BY-NC-SA 4.0 <http://creativecommons.org/licenses/by-nc-sa/4.0/>

SEE ALSO
    Manual (Online): <https://www.installminisap.com/minisap-plus/manual/>
    Manual (Local): /usr/local/share/com.installminisap.MiniSAPPlus.About.html

"
}

mspScriptExitNow() {
  if [ "${mspDoSilent}" == "0" ]; then
    if [ "${mspDoUninstall}" == "0" ] && [ "${mspDoUpdate}" == "0" ]; then
      echo -n "[MiniSAP+] Installation finished.

      Step 1, 2, 3 - Were just done by this script, the rest is up to you ...

  $(mspScriptNextSteps)

  "
    fi

    # Useful when './install.sh' is executed within an existing installations
    # .desktop files 'Exec=' action, for a 'MiniSAP Plus (+)' upgrade.
    if [ "${mspDoFinishWithSleep}" != "0" ]; then
      echo -n -e "\n[MiniSAP+] Finish timeout set for '${mspFinishWithSleep}' seconds. "
      read -s -t ${mspFinishWithSleep}
      echo ''
    fi
    if [ "${mspDoFinishWithEnter}" != "0" ] && [ "${mspDoQuiet}" == "0" ]; then
      echo -n -e "\n[MiniSAP+] Press [Enter] or [Return] to Finish. "
      read -s
      echo ''
    fi
  fi
}

#-------------------------------------------------------------------------------

# Default options ... Do NOT edit.
mspDoConfigFilePassword=0
mspDoConfigFileReleaseLifecycle=0
mspDoConfigFileUserName=0
mspDoFinishWithEnter=0
mspDoFinishWithSleep=0
mspDoHelp=0
mspDoNotRoot=0
mspDoQuiet=0
mspDoShellDebug=0
mspDoShellStrict=0
mspDoSilent=0
mspDoUninstall=0
mspDoUpdate=0
mspDoVerbose=0
mspDoVersion=0

# Default values ... Do NOT edit.
mspOSMiniSAPAdminPassword="";         # Always leave this one blank!
mspOSMiniSAPAdminPasswordLine="";     # Always leave this one blank!
mspOSMiniSAPAdminUserName="";         # Always leave this one blank!
mspOSMiniSAPAdminUserNameLine="";     # Always leave this one blank!
mspOSMiniSAPReleaseLifecycle="";      # Always leave this one blank!
mspOSMiniSAPReleaseLifecycleLine="";  # Always leave this one blank!
mspOSMiniSAPConfigurationOSAdminUserName="MiniSAP-OS-Admin-User-Name"
mspOSMiniSAPConfigurationOSAdminPassword="MiniSAP-OS-Admin-Password"
mspOSMiniSAPConfigurationReleaseLifecycle="MiniSAP-Plus-Release-Lifecycle"
mspScriptArgument=""
mspScriptArguments=()
mspScriptArgumentsInvalid=()
mspScriptArgumentsValid=()

# Defaults for installation update ... Do NOT edit.
mspUpdateConfigurationReleaseDownloadURL="MiniSAP-Plus-Release-Download-URL"
mspUpdateConfigurationReleaseLifecycle="MiniSAP-Plus-Release-Lifecycle"
mspUpdateConfigurationReleaseVersion="MiniSAP-Plus-Release-Version"
mspUpdateReleaseLifecycle=""
mspUpdateReleaseLifecycleFilename=""
mspUpdateReleaseLifecycleURL=""
mspUpdateReleaseLifecycleAlpha="alpha"
mspUpdateReleaseLifecycleAlphaFilename="${mspPackageName}-${mspUpdateReleaseLifecycleAlpha}.conf"
mspUpdateReleaseLifecycleAlphaURL="http://download.dataprofessionals.com.au/repository/learn/1.0/minisap/script/${mspUpdateReleaseLifecycleAlphaFilename}"
mspUpdateReleaseLifecycleBeta="beta"
mspUpdateReleaseLifecycleBetaFilename="${mspPackageName}-${mspUpdateReleaseLifecycleBeta}.conf"
mspUpdateReleaseLifecycleBetaURL="http://download.dataprofessionals.com.au/repository/learn/1.0/minisap/script/${mspUpdateReleaseLifecycleBetaFilename}"
mspUpdateReleaseLifecycleStable="stable"
mspUpdateReleaseLifecycleStableFilename="${mspPackageName}-${mspUpdateReleaseLifecycleStable}.conf"
mspUpdateReleaseLifecycleStableURL="http://download.dataprofessionals.com.au/repository/learn/1.0/minisap/script/${mspUpdateReleaseLifecycleStableFilename}"
mspUpdateReleaseLifecycleUnstable="unstable"
mspUpdateReleaseLifecycleUnstableFilename="${mspPackageName}-${mspUpdateReleaseLifecycleUnstable}.conf"
mspUpdateReleaseLifecycleUnstableURL="http://download.dataprofessionals.com.au/repository/learn/1.0/minisap/script/${mspUpdateReleaseLifecycleUnstableFilename}"

# Default configuration files ... Do NOT edit.
mspConfigFileOSAdminPassword=""
mspConfigFileOSAdminUserName=""
mspConfigFileReleaseLifecycle=""
mspFileListFQNErrorList=()
# These configuration items are for unit testing to force errors and abort.
# mspFileListFQN+=('' ' ' '.' '\' '/' '/*' '*' '~/' '~/*' '~/image.png'
#                  '\blah\blah/' './' './*' './image.png' '../'
#                  '../image.png' 'image.png')

# Default verbose output text ... Do Not Edit.
mspVerboseChGrp="[MiniSAP+] "
mspVerboseCopyOf="[MiniSAP+] copy of "
mspVerboseFileMkDir="[MiniSAP+] File directory. "
mspVerboseWGet="[MiniSAP+] "
mspVerboseMkDir="[MiniSAP+] /usr/local/bin/minisapplus/ directory. "
mspVerboseModeOf="[MiniSAP+] "
mspVerboseNewLine="\n"
mspVerboseOption="-v"
mspVerboseOptionWGet="--verbose"
mspVerbosePackageVersionMkDir="[MiniSAP+] /opt/<provider>/<package-version>/ directory. "
mspVerboseProviderMkDir="[MiniSAP+] /opt/<provider>/ directory. "
mspVerboseRemove="[MiniSAP+] "
mspVerboseSymLnk="[MiniSAP+] link for "
mspVerboseUnzip="[MiniSAP+] "

#-------------------------------------------------------------------------------
# Provider, Package, Directories and Files for default locations.
# Read Me: https://www.pathname.com/fhs/pub/fhs-2.3.html#OPTADDONAPPLICATIONSOFTWAREPACKAGES

# Default directories for storage and execution.
mspPackageVersionName="${mspPackageName}-${mspVersion}"
mspOptProviderDirectory="/opt/${mspProviderName}/"
mspOptPackageDirectory="/opt/${mspPackageName}/"
mspOptPackageLink="${mspOptPackageDirectory%\/}"
mspOptPackageVersionDirectory="/opt/${mspProviderName}/${mspPackageVersionName}/"
mspMiniSAPConfigurationLocalDirectory="/etc/${mspPackageName}/"
mspMiniSAPConfigurationLocalFQN="${mspMiniSAPConfigurationLocalDirectory}${mspPackageName}.conf"

# Capture and force current directory to be the location of this script.
mspScriptDirectory="$(dirname "$BASH_SOURCE")"
if [ "${mspScriptDirectory}" == "." ]; then
  mspScriptDirectory="$(pwd -P)";
fi
cd "${mspScriptDirectory}"
mspScriptDirectory="$(pwd -P)/"
mspScriptFilename="${0##*/}"

#-------------------------------------------------------------------------------
# Process the options and prepare for execution.

# Expand the compound options to be separated, for easier processing.
# Ensuring options start with '-', no outer quote wrapping.
# Forcing option values and arguments to get wrapped in outer quotes (').
mspScriptArguments=()
while (($#)); do
  case $1 in
    -[!-]?*) for ((i=1; i < ${#1}; i++)); do mspScriptArguments+=("-${1:i:1}"); done;;  # Expand '-abc' to '-a -b -c'.
    --?*=*)  mspScriptArguments+=("${1%%=*}" "'${1#*=}'");;  # Expand "--option={value}" to "--option 'value'".
    --?*)    mspScriptArguments+=("${1}");;
    --)      mspScriptArguments+=(--no-opt);;  # Dummy option for empty ones.
    *)       mspScriptArguments+=("${1@Q}");;
  esac
  shift
done
set -- "${mspScriptArguments[@]}";  # Put all expanded options back into official script array.

# Review each option, ignoring arguments and values ...
mspScriptArgumentsInvalid=()
mspScriptArgumentsValid=()
while (($#)); do
  mspScriptArgument="${1#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
  if [ "${mspScriptArgument:0:1}" == "-" ]; then
    if [ "${mspScriptArgument:0:2}" == "--" ]; then
      mspScriptArgument="${mspScriptArgument,,}";  # Make all lower case.
    fi
    case $mspScriptArgument in
         --debug)         mspDoShellDebug=1;      mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --finish-key)    mspDoFinishWithEnter=1; mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --finish-time)   mspDoFinishWithSleep=1; mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -h|--help)          mspDoHelp=1;            mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --not-root)      mspDoNotRoot=1;         mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -p|--password)      mspDoConfigFilePassword=1;   mspScriptArgumentsValid+=("${mspScriptArgument}");
                          mspScriptArgument="${2#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
                          if [ "${mspScriptArgument:0:1}" != "-" ]; then
                            mspOSMiniSAPAdminPassword="${mspScriptArgument}";
                            mspScriptArgumentsValid+=("${mspScriptArgument}");
                            shift;
                          fi;;
      -q|--quiet)         mspDoQuiet=1;           mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -s|--silent)        mspDoSilent=1;          mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --strict)        mspDoShellStrict=1;     mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --uninstall-now) mspDoUninstall=1;       mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --update-now)    mspDoUpdate=1;          mspScriptArgumentsValid+=("${mspScriptArgument}");
                          mspScriptArgument="${2#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
                          if [ "${mspScriptArgument:0:1}" != "-" ]; then
                            mspScriptArgument="${mspScriptArgument,,}";  # Make all lower case.
                            if [ "${mspScriptArgument}" == "" ]; then
                                mspUpdateReleaseLifecycle="";  #Allow it to be picked up from local config file.
                            elif [ "${mspScriptArgument}" == "${mspUpdateReleaseLifecycleAlpha}" ] ||
                                 [ "${mspScriptArgument}" == "${mspUpdateReleaseLifecycleBeta}" ] ||
                                 [ "${mspScriptArgument}" == "${mspUpdateReleaseLifecycleStable}" ] ||
                                 [ "${mspScriptArgument}" == "${mspUpdateReleaseLifecycleUnstable}" ]; then
                                mspUpdateReleaseLifecycle="${mspScriptArgument}";
                                mspScriptArgumentsValid+=("${mspScriptArgument}");
                            else
                              mspScriptArgumentsInvalid+=("${mspScriptArgument@Q}");
                            fi
                            shift;
                          fi;;
      -u|--username)      mspDoConfigFileUserName=1;   mspScriptArgumentsValid+=("${mspScriptArgument}");
                          mspScriptArgument="${2#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
                          if [ "${mspScriptArgument:0:1}" != "-" ]; then
                            mspOSMiniSAPAdminUserName="${mspScriptArgument}";
                            mspScriptArgumentsValid+=("${mspScriptArgument}");
                            shift;
                          fi;;
      -v|--verbose)       mspDoVerbose=1;         mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --version)       mspDoVersion=1;         mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --no-opt)        shift;;  # Ignore it.
      *)                  mspScriptArgumentsInvalid+=("${mspScriptArgument@Q}");;
    esac
  else
    mspScriptArgumentsValid+=("${mspScriptArgument}")
  fi
  shift
done
set -- "${mspScriptArgumentsValid[@]}";  # Put all valid arguments back into official script array.

# Process '--help' option before complaining about invalid options.
if [ "${mspDoHelp}" != 0 ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    mspScriptUsage
  fi
  mspScriptExitNow
  exit 0;  # 0 is 'Ok'.
fi

# Enforce use of only valid options.
if [ "${#mspScriptArgumentsInvalid[@]}" != 0 ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    echo "[MiniSAP+] Execution aborted."
    echo "[MiniSAP+] Invalid Options: ${mspScriptArgumentsInvalid[@]}"
    echo "[MiniSAP+] Use '${mspScriptDirectory}${mspScriptFilename} --help' option for full details."
  fi
  mspScriptExitNow
  exit 1;  # 1 is 'Error - Invalid arguments passed to script.'.
fi

#-------------------------------------------------------------------------------
# Set execution modes for shell.
# Reference URL: http://linuxcommand.org/lc3_man_pages/seth.html

if [ "${mspDoShellStrict}" != "0" ]; then
  set -o nounset
  if [ "$?" = 0 ] && [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Shell Execution: 'strict' mode enabled."
  fi
fi
set -o pipefail;  # Return the highest return code of piped commands.
# if [ "$?" = 0 ] && [ "${mspDoVerbose}" != "0" ]; then
#   echo "[MiniSAP+] Shell Execution: 'pipeline' mode enabled."
# fi
if [ "${mspDoShellDebug}" != "0" ]; then
  set -x
  if [ "$?" = 0 ] && [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Shell Execution: 'debug' mode enabled."
  fi
fi

#-------------------------------------------------------------------------------

if [ "${mspDoVerbose}" == "0" ] || [ "${mspDoSilent}" != "0" ]; then
  mspDoVerbose=0;  # Option '--silent' turns '--verbose' option off.
  mspVerboseChGrp=""
  mspVerboseCopyOf=""
  mspVerboseFileMkDir=""
  mspVerboseWGet=""
  mspVerboseMkDir=""
  mspVerboseModeOf=""
  mspVerboseNewLine=""
  mspVerboseOption=""
  mspVerboseOptionWGet="--no-verbose"
  mspVerbosePackageVersionMkDir=""
  mspVerboseProviderMkDir=""
  mspVerboseRemove=""
  mspVerboseSymLnk=""
  mspVerboseUnzip=""
fi



################################################################################
# [START EDITING] Add the custom script code below this line.
################################################################################

#-------------------------------------------------------------------------------
# Process version options.

if [ "${mspDoVersion}" != "0" ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    echo "${mspVersion}";
  fi
  mspScriptExitNow
  exit 0;  # 0 is 'Ok'.
fi

#-------------------------------------------------------------------------------
# Installation healthcheck.

if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Execution healthcheck starting ..."; fi

# Get Configuration - MiniSAP OS Password to be used.
mspConfigFileOSAdminPassword=""
mspOSMiniSAPAdminPasswordLine="$(grep --no-messages -i -e "^${mspOSMiniSAPConfigurationOSAdminPassword}=.*" ${mspMiniSAPConfigurationLocalFQN})"
if [ "${mspOSMiniSAPAdminPasswordLine}" != "" ]; then
  mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPAdminPasswordLine//\"/\'}";  # Replace all " with '.
  mspConfigFileOSAdminPassword="${mspOSMiniSAPAdminPasswordLine#*=}"
  mspConfigFileOSAdminPassword="${mspConfigFileOSAdminPassword#\'}"; mspConfigFileOSAdminPassword="${mspConfigFileOSAdminPassword%\'}";
fi
if [ "${mspOSMiniSAPAdminPassword}" == "" ]; then
  mspOSMiniSAPAdminPassword="${mspConfigFileOSAdminPassword}"
  if [ "${mspOSMiniSAPAdminPassword}" == "" ]; then
    mspOSMiniSAPAdminPassword="${mspOSMiniSAPAdminPasswordDefault}"
  fi
fi
mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPConfigurationOSAdminPassword}=\"${mspOSMiniSAPAdminPassword}\""
mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPAdminPasswordLine//\'/\\\'}";  # Escape ' for sed execution.
if [ "${mspOSMiniSAPAdminPassword}" != "" ]; then
  mspDoConfigFilePassword=1
fi

# Get Configuration - MiniSAP OS User Name to be used.
mspConfigFileOSAdminUserName=""
mspOSMiniSAPAdminUserNameLine="$(grep --no-messages -i -e "^${mspOSMiniSAPConfigurationOSAdminUserName}=.*" ${mspMiniSAPConfigurationLocalFQN})"
if [ "${mspOSMiniSAPAdminUserNameLine}" != "" ]; then
  mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPAdminUserNameLine//\"/\'}";  # Replace all " with '.
  mspConfigFileOSAdminUserName="${mspOSMiniSAPAdminUserNameLine#*=}"
  mspConfigFileOSAdminUserName="${mspConfigFileOSAdminUserName#\'}"; mspConfigFileOSAdminUserName="${mspConfigFileOSAdminUserName%\'}";
fi
if [ "${mspOSMiniSAPAdminUserName}" == "" ]; then
  mspOSMiniSAPAdminUserName="${mspConfigFileOSAdminUserName}"
  if [ "${mspOSMiniSAPAdminUserName}" == "" ]; then
    mspOSMiniSAPAdminUserName="${mspOSMiniSAPAdminUserNameDefault}"
    # Later: Once you know the .desktop 'TryExec=?' touch file for execution to
    #        detect existance of HANA stack install (not ABAP stack) then add
    #        if/then/else here for ... mspOSMiniSAPAdminUserName="HXEADM".
  fi
fi
mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPConfigurationOSAdminUserName}=\"${mspOSMiniSAPAdminUserName}\""
mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPAdminUserNameLine//\'/\\\'}";  # Escape ' for sed execution.
if [ "${mspOSMiniSAPAdminUserName}" != "" ]; then
  mspDoConfigFileUserName=1
fi

# Get Configuration - MiniSAP Release Lifecycle to be used.
mspConfigFileReleaseLifecycle=""
mspOSMiniSAPReleaseLifecycleLine="$(grep --no-messages -i -e "^${mspOSMiniSAPConfigurationReleaseLifecycle}=.*" ${mspMiniSAPConfigurationLocalFQN})"
if [ "${mspOSMiniSAPReleaseLifecycleLine}" != "" ]; then
  mspOSMiniSAPReleaseLifecycleLine="${mspOSMiniSAPReleaseLifecycleLine//\"/\'}";  # Replace all " with '.
  mspConfigFileReleaseLifecycle="${mspOSMiniSAPReleaseLifecycleLine#*=}"
  mspConfigFileReleaseLifecycle="${mspConfigFileReleaseLifecycle#\'}"; mspConfigFileReleaseLifecycle="${mspConfigFileReleaseLifecycle%\'}";
fi
if [ "${mspOSMiniSAPReleaseLifecycle}" == "" ]; then
  mspOSMiniSAPReleaseLifecycle="${mspConfigFileReleaseLifecycle}"
  if [ "${mspOSMiniSAPReleaseLifecycle}" == "" ]; then
    mspOSMiniSAPReleaseLifecycle="${mspOSMiniSAPReleaseLifecycleDefault}"
  fi
fi
mspOSMiniSAPReleaseLifecycle="${mspOSMiniSAPReleaseLifecycle,,}";  # Make all lower case.
mspOSMiniSAPReleaseLifecycleLine="${mspOSMiniSAPConfigurationReleaseLifecycle}=\"${mspOSMiniSAPReleaseLifecycle}\""
mspOSMiniSAPReleaseLifecycleLine="${mspOSMiniSAPReleaseLifecycleLine//\'/\\\'}";  # Escape ' for sed execution.
if [ "${mspOSMiniSAPReleaseLifecycle}" != "" ]; then
  mspDoConfigFileReleaseLifecycle=1
fi

# Check MiniSAP Release Lifecycle to be used for 'Update' capability.
if [ "${mspUpdateReleaseLifecycle}" == "" ]; then
  mspUpdateReleaseLifecycle="${mspOSMiniSAPReleaseLifecycle}"
fi

# Output variable configuration to be executed.
if [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] Execution Directory ..... : ${mspScriptDirectory}"
  echo "[MiniSAP+] Execution Filename ...... : ${mspScriptFilename}"
  echo "[MiniSAP+] Provider Directory ...... : ${mspOptProviderDirectory}"
  echo "[MiniSAP+] Package Directory ....... : ${mspOptPackageDirectory}"
  echo "[MiniSAP+] Package Version Directory : ${mspOptPackageVersionDirectory}"
  echo "[MiniSAP+] Local Config Directory .. : ${mspMiniSAPConfigurationLocalDirectory}"
  echo "[MiniSAP+] Config OS Admin User Name : ${mspOSMiniSAPAdminUserName}"
  echo "[MiniSAP+] Config OS Admin Password. : ${mspOSMiniSAPAdminPassword}"
  echo "[MiniSAP+] Config Release Lifecycle. : ${mspOSMiniSAPReleaseLifecycle}"
  echo "[MiniSAP+] Not Root Option ......... : ${mspDoNotRoot}"
  echo "[MiniSAP+] Uninstall Option ........ : ${mspDoUninstall}"
  echo "[MiniSAP+] Update Option ........... : ${mspDoUpdate} for '${mspUpdateReleaseLifecycle}'."
fi

# Check the file configuration array for valid FQNs.
# Seriously ... Do NOT remove this because the bulk file maintenance commands
#               that come next are dangerous to the health of the operating
#               system. Executing as a root group user has responsibilities
#               and these checks help to stop the 'you are a stupid dumb ass'
#               obvious stuff from happening (but life has no guarantees!).
mspFileListFQNErrorList=()
mspFileListFQNCheckList=($mspFileListFQN[@])
mspFileListFQNCheckList+=($mspFileListFQNDeleteMeHousekeeping[@])
for mspFileFQN in "${mspFileListFQNCheckList[@]}"
do
  case $mspFileFQN in
    *\\*) mspFileListFQNErrorList+=("Error in '${mspFileFQN}': Use of '\' character not allowed, use '/' as the directory separator.");;
    /)    mspFileListFQNErrorList+=("Error in '${mspFileFQN}': Root directory! Really? No, forget-about-it.");;
  esac

  mspFileName="${mspFileFQN##*/}"
  case $mspFileName in
    '*') mspFileListFQNErrorList+=("Error in '${mspFileFQN}': All files '*' is too dangerous, use a filename prefix like 'minisap*'.");;
    '')  mspFileListFQNErrorList+=("Error in '${mspFileFQN}': File name can not be blank, specify a filename prefix like 'minisap*'.");;
  esac

  mspFileDirectory="$(dirname "$mspFileFQN")/"
  case $mspFileDirectory in
    /)      mspFileListFQNErrorList+=("Error in '${mspFileFQN}': Root directory! Really? No, forget-about-it.");;
    \~/*)   mspFileListFQNErrorList+=("Error in '${mspFileFQN}': Home '~/' directory of a root group user not allowed; use full path like '/usr/local/bin/minisap*'.");;
    .?*)    mspFileListFQNErrorList+=("Error in '${mspFileFQN}': './' relative directory not allowed; use full path like '/usr/local/bin/minisap*'.");;
    /etc/*) ;;  # Target directory is ok to use.
    /tmp/*) ;;  # Target directory is ok to use.
    /usr/*) ;;  # Target directory is ok to use.
    /var/*) ;;  # Target directory is ok to use.
    *)      mspFileListFQNErrorList+=("Error in '${mspFileFQN}': Target '${mspFileDirectory}' directory not allowed.");;
  esac
done
if [ "${#mspFileListFQNErrorList[@]}" != "0" ] && [ "${mspDoUpdate}" == "0" ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    echo "[MiniSAP+] Installation aborted."
    echo "[MiniSAP+] Internal variable 'mspFileListFQN=()' array needs attention."
    for ((i=0; i < ${#mspFileListFQNErrorList[@]}; i++)); do
      echo "[MiniSAP+] ${mspFileListFQNErrorList[i]}"
    done
  fi
  mspScriptExitNow
  exit 3;  # 3 is 'Error - Internal variable 'mspFileListFQN=()' array needs attention.'.
fi

if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Execution healthcheck finished."; fi

#-------------------------------------------------------------------------------
# This installation script must be executed as 'root' using 'sudo' command.
# Allow other options to be processed before we error out lack of root access.

mspExecutingUser=$(whoami)
if [ "${mspDoNotRoot}" == "0" ] && [ "${mspExecutingUser}" != "root" ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    echo "[MiniSAP+] Installation aborted."
    echo "[MiniSAP+] Must execute as 'root' user or use '--not-root' option."
    echo "[MiniSAP+] Use '${mspScriptDirectory}${mspScriptFilename} --help' option for full details."
  fi
  mspScriptExitNow
  exit 2;  # 2 is 'Error - Expecting root user access for installation.'.
fi
if [ "${mspDoNotRoot}" != "0" ] && [ "${mspExecutingUser}" != "root" ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    echo "[MiniSAP+] Honouring the '--not-root' option ... continuing."
  fi
fi

#-------------------------------------------------------------------------------
# Lets do it ... copy this entire installer into the /opt/ add-on packages.

if [ "${mspDoUninstall}" == "0" ] && [ "${mspDoUpdate}" == "0" ]; then

  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] /opt/ for add-on packages starting ..."; fi

  if [ "${mspScriptDirectory:0:4}" != "/opt" ]; then

    # Create /opt/<provider>/<package-version>/.
    echo -n "${mspVerboseProviderMkDir}$(mkdir ${mspVerboseOption} --parents            ${mspOptProviderDirectory})"; echo -ne "${mspVerboseNewLine}";
    echo -n "${mspVerboseModeOf}$(       chmod ${mspVerboseOption} --preserve-root 0775 ${mspOptProviderDirectory})"; echo -ne "${mspVerboseNewLine}";
    echo -n "${mspVerboseChGrp}$(        chgrp ${mspVerboseOption} root                 ${mspOptProviderDirectory})"; echo -ne "${mspVerboseNewLine}";

    echo -n "${mspVerbosePackageVersionMkDir}$(mkdir ${mspVerboseOption} --parents            ${mspOptPackageVersionDirectory})"; echo -ne "${mspVerboseNewLine}";
    echo -n "${mspVerboseModeOf}$(             chmod ${mspVerboseOption} --preserve-root 0775 ${mspOptPackageVersionDirectory})"; echo -ne "${mspVerboseNewLine}";
    echo -n "${mspVerboseChGrp}$(              chgrp ${mspVerboseOption} root                 ${mspOptPackageVersionDirectory})"; echo -ne "${mspVerboseNewLine}";

    # Create /opt/<package> symbolic link to /opt/<provider>/<package-version>/.
    echo -n "${mspVerboseSymLnk}$(ln ${mspVerboseOption} --symbolic --force -T ${mspOptPackageVersionDirectory} ${mspOptPackageLink})"; echo -ne "${mspVerboseNewLine}";

    # Ensure /opt/<provider>/<package-version>/ directory is empty.
    echo -n "${mspVerboseRemove}$(rm ${mspVerboseOption} --recursive --dir --force --preserve-root ${mspOptPackageVersionDirectory}/*)"; echo -ne "${mspVerboseNewLine}";

    # Copy current installer files from './' into /opt/<provider>/<package-version>/.
    echo -n "${mspVerboseCopyOf}$(cp    ${mspVerboseOption} -H --recursive --force --target-directory=${mspOptPackageVersionDirectory} ${mspScriptDirectory}/*)"; echo -ne "${mspVerboseNewLine}";
    echo -n "${mspVerboseModeOf}$(chmod ${mspVerboseOption} --recursive --preserve-root 0775 ${mspOptPackageVersionDirectory})"; echo -ne "${mspVerboseNewLine}";
    echo -n "${mspVerboseChGrp}$( chgrp ${mspVerboseOption} -H --recursive --preserve-root root ${mspOptPackageVersionDirectory})"; echo -ne "${mspVerboseNewLine}";

  else
    echo "[MiniSAP+] /opt/ package directory is already being used by installer."
  fi

  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] /opt/ for add-on packages finished."; fi
fi

#-------------------------------------------------------------------------------
# Lets do it ... assuming use of Gnome desktop in OpenSUSE 15 (Leap).
# ... because this script is a companion for <https://www.installminisap.com/>.

if [ "${mspDoUninstall}" == "0" ] && [ "${mspDoUpdate}" == "0" ]; then

  if [ "${mspDoSilent}" == "0" ]; then echo "[MiniSAP+] Installation starting ..."; fi

  for mspFileFQN in "${mspFileListFQN[@]}"
  do
    mspFileName="${mspFileFQN##*/}"
    mspFileDirectory="$(dirname "$mspFileFQN")/"

    # Last chance paronoid check for 'Relative directories not allowed'.
    if [ "${mspFileFQN}" == "" ] ||
       [ "${mspFileFQN}" == "./" ] ||
       [ "${mspFileFQN}" == "../" ] ||
       [ "${mspFileName}" == "" ] ||
       [ "${mspFileName}" == "*" ] ||
       [ "${mspFileName}" == "./" ] ||
       [ "${mspFileName}" == "../" ] ||
       [ "${mspFileDirectory}" == "" ] ||
       [ "${mspFileDirectory}" == "/" ] ||
       [ "${mspFileDirectory}" == "." ] ||
       [ "${mspFileDirectory}" == ".." ] ||
       [ "${mspFileDirectory}" == "./" ] ||
       [ "${mspFileDirectory}" == "../" ]; then
      if [ "${mspDoSilent}" == "0" ]; then
        echo "[MiniSAP+] Installation aborted."
        echo "[MiniSAP+] Internal variable 'mspFileListFQN=()' array STILL needs attention."
        echo "[MiniSAP+] Error in '${mspFileFQN}': './' relative directory not allowed; use full path like '/usr/local/bin/minisap*'."
      fi
      mspScriptExitNow
      exit 4;  # 4 is 'Error - Internal variable 'mspFileListFQN=()' array STILL needs attention.'.
    fi

    # Create .../<directory>/<package-file>.
    if [ "${mspDoVerbose}" != "0" ]; then
      echo "[MiniSAP+] Installing '${mspFileFQN}' in '${mspFileDirectory}'."
    fi

    # Create a package directory if the file's FQN has the package name in it.
    if [ -z "${mspFileDirectory##*$mspPackageName*}" ] ;then
      echo -n "${mspVerboseFileMkDir}$(mkdir ${mspVerboseOption} --parents            ${mspFileDirectory})"; echo -ne "${mspVerboseNewLine}";
      echo -n "${mspVerboseModeOf}$(   chmod ${mspVerboseOption} --preserve-root 0775 ${mspFileDirectory})"; echo -ne "${mspVerboseNewLine}";
      echo -n "${mspVerboseChGrp}$(    chgrp ${mspVerboseOption} root                 ${mspFileDirectory})"; echo -ne "${mspVerboseNewLine}";
    fi

    # Process the file ... allow for --not-root option to still allow root group full access.
    mspFileMode="0664";  # rw-rw-r-- ... read & write for user/group but no execute.
    if [ "${mspFileFQN: -3}" == ".sh" ] ;then
      mspFileMode="0775";  # rwxrwxr-x ...read & write & execute for user/group but no write for others.
    fi
    echo -n "${mspVerboseCopyOf}$(  cp  ${mspVerboseOption} -f ${mspOptPackageVersionDirectory}${mspFileFQN} ${mspFileDirectory})"; echo -ne "${mspVerboseNewLine}";
    echo -n "${mspVerboseModeOf}$(chmod ${mspVerboseOption} --preserve-root ${mspFileMode}     ${mspFileFQN})";                     echo -ne "${mspVerboseNewLine}";
    echo -n "${mspVerboseChGrp}$( chgrp ${mspVerboseOption} root                               ${mspFileFQN})";                     echo -ne "${mspVerboseNewLine}";

    # Create '*.conf' file from '*-example.conf' file, if not exist and is in a package directory.
    if [ -z "${mspFileDirectory##*$mspPackageName*}" ] &&
       [ "${mspFileFQN: -13}" == "-example.conf" ]; then
      mspFileConfigurationFQN="${mspFileFQN//-example.conf/.conf}"
      if [ -f "${mspFileConfigurationFQN}" ] ;then
        if [ "${mspDoVerbose}" != "0" ]; then
          echo "[MiniSAP+] Config '${mspFileConfigurationFQN}' file already exists."
        fi
      else
        if [ "${mspDoVerbose}" != "0" ]; then
          echo "[MiniSAP+] Config '${mspFileConfigurationFQN}' file needs to be created."
        fi
        echo -n "${mspVerboseCopyOf}$(cp ${mspVerboseOption} -f ${mspFileFQN} ${mspFileConfigurationFQN})"; echo -ne "${mspVerboseNewLine}";
      fi
      if [ -f "${mspFileConfigurationFQN}" ] ;then
        if [ "${mspDoVerbose}" != "0" ]; then
          echo "[MiniSAP+] Config '${mspFileDirectory}' directory permissions need to be enforced."
        fi
        # Enable directory write for all, allow 'sed' command to function using temporary files.
        echo -n "${mspVerboseModeOf}$(chmod ${mspVerboseOption} --preserve-root 0777 ${mspFileDirectory})"; echo -ne "${mspVerboseNewLine}";
        if [ "${mspDoVerbose}" != "0" ]; then
          echo "[MiniSAP+] Config '${mspFileConfigurationFQN}' file permissions need to be enforced."
        fi
        echo -n "${mspVerboseModeOf}$(chmod ${mspVerboseOption} --preserve-root 0777 ${mspFileConfigurationFQN})"; echo -ne "${mspVerboseNewLine}";
        echo -n "${mspVerboseChGrp}$( chgrp ${mspVerboseOption} root                 ${mspFileConfigurationFQN})"; echo -ne "${mspVerboseNewLine}";
      fi
    fi

    # Load the new .desktop file without rebooting ... alledegly ???
    if [ "${mspFileFQN: -8}" == ".desktop" ] ;then
      xdg-desktop-menu install --mode system --novendor "${mspFileFQN}"
      if [ "$?" = 0 ] && [ "${mspDoVerbose}" != "0" ]; then
        echo "[MiniSAP+] Desktop '${mspFileFQN}' file was installed in application menu."
      fi
    fi
  done
fi

#-------------------------------------------------------------------------------
# Update '/etc/minisap-plus/minisap-plus.conf' scripts to edit the default MiniSAP Admin.

if [ "${mspDoUninstall}" == "0" ] && [ "${mspDoUpdate}" == "0" ]; then

  # Update 'MiniSAP-OS-Admin-User-Name=' variable defeult value in target scripts.
  if [ "${mspDoConfigFileUserName}" != "0" ]; then
    if [ "${mspDoVerbose}" != "0" ]; then
      echo "[MiniSAP+] Config - Admin User Name update starting ..."
    fi
    # echo -n "$(sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationOSAdminUserName}=.*/${mspOSMiniSAPAdminUserNameLine}/I" ${mspMiniSAPConfigurationLocalFQN})"; echo -ne "${mspVerboseNewLine}";
    grep --silent --no-messages -i -e "^${mspOSMiniSAPConfigurationOSAdminUserName}=.*" ${mspMiniSAPConfigurationLocalFQN} && sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationOSAdminUserName}=.*/${mspOSMiniSAPAdminUserNameLine}/I" ${mspMiniSAPConfigurationLocalFQN} || echo -e "\n${mspOSMiniSAPAdminUserNameLine}" >> ${mspMiniSAPConfigurationLocalFQN}
    if [ "${mspDoVerbose}" != "0" ]; then
      echo "[MiniSAP+] Config - Admin User Name update finished."
    fi
  elif [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Admin User Name update ... will be skipped."
  fi

  # Update 'MiniSAP-OS-Admin-Password=' variable defeult value in target scripts.
  if [ "${mspDoConfigFilePassword}" != "0" ]; then
    if [ "${mspDoVerbose}" != "0" ]; then
      echo "[MiniSAP+] Config - Admin Password update starting ..."
    fi
    # echo -n "$(sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationOSAdminPassword}=.*/${mspOSMiniSAPAdminPasswordLine}/I" ${mspMiniSAPConfigurationLocalFQN})"; echo -ne "${mspVerboseNewLine}";
    grep --silent --no-messages -i -e "^${mspOSMiniSAPConfigurationOSAdminPassword}=.*" ${mspMiniSAPConfigurationLocalFQN} && sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationOSAdminPassword}=.*/${mspOSMiniSAPAdminPasswordLine}/I" ${mspMiniSAPConfigurationLocalFQN} || echo -e "\n${mspOSMiniSAPAdminPasswordLine}" >> ${mspMiniSAPConfigurationLocalFQN}
    if [ "${mspDoVerbose}" != "0" ]; then
      echo "[MiniSAP+] Config - Admin Password update finished."
    fi
  elif [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Admin Password update ... will be skipped."
  fi

  # Update 'MiniSAP-Plus-Release-Lifecycle=' variable defeult value in target scripts.
  if [ "${mspDoConfigFileReleaseLifecycle}" != "0" ]; then
    if [ "${mspDoVerbose}" != "0" ]; then
      echo "[MiniSAP+] Config - Release Lifecycle update starting ..."
    fi
    # echo -n "$(sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationReleaseLifecycle}=.*/${mspOSMiniSAPReleaseLifecycleLine}/I" ${mspMiniSAPConfigurationLocalFQN})"; echo -ne "${mspVerboseNewLine}";
    grep --silent --no-messages -i -e "^${mspOSMiniSAPConfigurationReleaseLifecycle}=.*" ${mspMiniSAPConfigurationLocalFQN} && sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationReleaseLifecycle}=.*/${mspOSMiniSAPReleaseLifecycleLine}/I" ${mspMiniSAPConfigurationLocalFQN} || echo -e "\n${mspOSMiniSAPReleaseLifecycleLine}" >> ${mspMiniSAPConfigurationLocalFQN}
    if [ "${mspDoVerbose}" != "0" ]; then
      echo "[MiniSAP+] Config - Release Lifecycle update finished."
    fi
  elif [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Release Lifecycle update ... will be skipped."
  fi
fi

#-------------------------------------------------------------------------------
# Goodbye ... uninstall it now!!!

if [ "${mspDoUninstall}" != "0" ]; then

  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Uninstall starting ..."; fi

  mspFileListFQNUninstallList=()
  mspFileListFQNUninstallList=("${mspFileListFQN[@]}")
  mspFileListFQNUninstallList+=("${mspFileListFQNDeleteMeHousekeeping[@]}")
  for mspFileFQN in "${mspFileListFQNUninstallList[@]}"
  do
    mspFileName="${mspFileFQN##*/}"
    mspFileDirectory="$(dirname "$mspFileFQN")/"

    # Last chance paronoid check for 'Relative directories not allowed'.
    if [ "${mspFileFQN}" == "" ] ||
       [ "${mspFileFQN}" == "./" ] ||
       [ "${mspFileFQN}" == "../" ] ||
       [ "${mspFileName}" == "" ] ||
       [ "${mspFileName}" == "*" ] ||
       [ "${mspFileName}" == "./" ] ||
       [ "${mspFileName}" == "../" ] ||
       [ "${mspFileDirectory}" == "" ] ||
       [ "${mspFileDirectory}" == "/" ] ||
       [ "${mspFileDirectory}" == "." ] ||
       [ "${mspFileDirectory}" == ".." ] ||
       [ "${mspFileDirectory}" == "./" ] ||
       [ "${mspFileDirectory}" == "../" ]; then
      if [ "${mspDoSilent}" == "0" ]; then
        echo "[MiniSAP+] Uninstall aborted."
        echo "[MiniSAP+] Internal variable 'mspFileListFQN=()' array and 'mspFileListFQNDeleteMeHousekeeping=()' STILL needs attention."
        echo "[MiniSAP+] Error in '${mspFileFQN}': './' relative directory not allowed; use full path like '/usr/local/bin/minisap*'."
      fi
      mspScriptExitNow
      exit 5;  # 5 is 'Error - Internal variable 'mspFileListFQN=()' array and 'mspFileListFQNDeleteMeHousekeeping=()' STILL needs attention.'.
    fi

    # Delete files match the files FQN.
    if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Housekeeping for '${mspFileFQN}' ..."; fi
    echo -n "${mspVerboseRemove}$(rm ${mspVerboseOption} --force --preserve-root ${mspFileFQN})"; echo -ne "${mspVerboseNewLine}";

    # Delete a directory if the file's FQN directory has the package name in it.
    if [ -z "${mspFileDirectory##*$mspPackageName*}" ] ;then
      if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Housekeeping for '${mspFileDirectory}' ..."; fi
      echo -n "${mspVerboseRemove}$(rm ${mspVerboseOption} --recursive --dir --force --preserve-root ${mspFileDirectory})"; echo -ne "${mspVerboseNewLine}";
    fi
  done

  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Uninstall finished."; fi
  mspScriptExitNow
  exit 0;  # 0 is 'Ok'.
fi

#-------------------------------------------------------------------------------
# Hello ... download and install latest version now!!!

if [ "${mspDoUpdate}" != "0" ]; then

  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Update starting ..."; fi

  # - Grab latest repository configuration file '*-{{lifecycle}}.conf' from the internet.
  case $mspUpdateReleaseLifecycle in
    "${mspUpdateReleaseLifecycleAlpha}")    mspUpdateReleaseLifecycleFilename="${mspUpdateReleaseLifecycleAlphaFilename}"
                                            mspUpdateReleaseLifecycleURL="${mspUpdateReleaseLifecycleAlphaURL}";;
    "${mspUpdateReleaseLifecycleBeta}")     mspUpdateReleaseLifecycleFilename="${mspUpdateReleaseLifecycleBetaFilename}"
                                            mspUpdateReleaseLifecycleURL="${mspUpdateReleaseLifecycleBetaURL}";;
    "${mspUpdateReleaseLifecycleUnstable}") mspUpdateReleaseLifecycleFilename="${mspUpdateReleaseLifecycleUnstableFilename}"
                                            mspUpdateReleaseLifecycleURL="${mspUpdateReleaseLifecycleUnstableURL}";;
    *)                                      mspUpdateReleaseLifecycleFilename="${mspUpdateReleaseLifecycleStableFilename}"
                                            mspUpdateReleaseLifecycle="${mspUpdateReleaseLifecycleStable}"
                                            mspUpdateReleaseLifecycleURL="${mspUpdateReleaseLifecycleStableURL}";;
  esac
  mspMiniSAPConfigurationLocalReleaseLifecycleFilename="${mspMiniSAPConfigurationLocalDirectory}${mspUpdateReleaseLifecycleFilename}"
  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Update '${mspMiniSAPConfigurationLocalReleaseLifecycleFilename}' from '${mspUpdateReleaseLifecycleURL}' ..."; fi
  echo -n "${mspVerboseWGet}"; echo -n "$(wget ${mspVerboseOptionWGet} --tries=1 --no-cache --no-cookies --ignore-length --compression=none --output-document=${mspMiniSAPConfigurationLocalReleaseLifecycleFilename} ${mspUpdateReleaseLifecycleURL})"; echo -ne "${mspVerboseNewLine}";

  # Get Configuration - Update MiniSAP Release Lifecycle in downloaded '*.conf' file.
  mspUpdateConfigFileReleaseLifecycle=""
  mspConfigLine="$(grep --no-messages -i -e "^${mspUpdateConfigurationReleaseLifecycle}=.*" ${mspMiniSAPConfigurationLocalReleaseLifecycleFilename})"
  if [ "${mspConfigLine}" != "" ]; then
    mspConfigLine="${mspConfigLine//\"/\'}";  # Replace all " with '.
    mspUpdateConfigFileReleaseLifecycle="${mspConfigLine#*=}"
    mspUpdateConfigFileReleaseLifecycle="${mspUpdateConfigFileReleaseLifecycle#\'}"; mspUpdateConfigFileReleaseLifecycle="${mspUpdateConfigFileReleaseLifecycle%\'}";
  fi
  mspUpdateConfigFileReleaseLifecycle="${mspUpdateConfigFileReleaseLifecycle,,}";  # Make all lower case.

  # Get Configuration - Update MiniSAP Release Version in downloaded '*.conf' file.
  mspUpdateConfigFileReleaseVersion=""
  mspConfigLine="$(grep --no-messages -i -e "^${mspUpdateConfigurationReleaseVersion}=.*" ${mspMiniSAPConfigurationLocalReleaseLifecycleFilename})"
  if [ "${mspConfigLine}" != "" ]; then
    mspConfigLine="${mspConfigLine//\"/\'}";  # Replace all " with '.
    mspUpdateConfigFileReleaseVersion="${mspConfigLine#*=}"
    mspUpdateConfigFileReleaseVersion="${mspUpdateConfigFileReleaseVersion#\'}"; mspUpdateConfigFileReleaseVersion="${mspUpdateConfigFileReleaseVersion%\'}";
  fi
  mspUpdateConfigFileReleaseVersion="${mspUpdateConfigFileReleaseVersion,,}";  # Make all lower case.

  # Get Configuration - Update MiniSAP Release DownloadURL in downloaded '*.conf' file.
  mspUpdateConfigFileReleaseDownloadURL=""
  mspConfigLine="$(grep --no-messages -i -e "^${mspUpdateConfigurationReleaseDownloadURL}=.*" ${mspMiniSAPConfigurationLocalReleaseLifecycleFilename})"
  if [ "${mspConfigLine}" != "" ]; then
    mspConfigLine="${mspConfigLine//\"/\'}";  # Replace all " with '.
    mspUpdateConfigFileReleaseDownloadURL="${mspConfigLine#*=}"
    mspUpdateConfigFileReleaseDownloadURL="${mspUpdateConfigFileReleaseDownloadURL#\'}"; mspUpdateConfigFileReleaseDownloadURL="${mspUpdateConfigFileReleaseDownloadURL%\'}";
  fi
  mspUpdateConfigFileReleaseDownloadURL="${mspUpdateConfigFileReleaseDownloadURL,,}";  # Make all lower case.

  # echo "mspUpdateConfigFileReleaseLifecycle=${mspUpdateConfigFileReleaseLifecycle}"
  # echo "mspUpdateConfigFileReleaseVersion=${mspUpdateConfigFileReleaseVersion}"
  # echo "mspUpdateConfigFileReleaseDownloadURL=${mspUpdateConfigFileReleaseDownloadURL}"

  # - Download file into /opt/minisap.dataprofessionals.com.au/ directory.
  mspUpdateLocalCompressedFilename="${mspUpdateConfigFileReleaseDownloadURL##*/}"
  mspUpdateLocalCompressedFQN="${mspOptProviderDirectory}${mspUpdateLocalCompressedFilename}"
  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Update '${mspUpdateLocalCompressedFQN}' from '${mspUpdateConfigFileReleaseDownloadURL}' ..."; fi
  echo -n "${mspVerboseWGet}"; echo -n "$(wget ${mspVerboseOptionWGet} --tries=1 --no-cache --no-cookies --ignore-length --compression=none --output-document=${mspUpdateLocalCompressedFQN} ${mspUpdateConfigFileReleaseDownloadURL})"; echo -ne "${mspVerboseNewLine}";

  # Decompress the installation file into the /opt/minisap.dataprofessionals.com.au/ directory.
  # Compressed file content should have all files stored within a parent directory like './minisap-plus-{{version}}/'.
  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Decompress '${mspUpdateLocalCompressedFQN}' ..."; fi
  echo -n "${mspVerboseUnzip}$(unzip ${mspVerboseOption} -o ${mspUpdateLocalCompressedFQN})"; echo -ne "${mspVerboseNewLine}";

  # - Delete the compressed file from /opt/minisap.dataprofessionals.com.au/ directory.
  # if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Housekeeping for '${mspUpdateLocalCompressedFQN}' ..."; fi
  # echo -n "${mspVerboseRemove}$(rm ${mspVerboseOption} --force --preserve-root ${mspUpdateLocalCompressedFQN})"; echo -ne "${mspVerboseNewLine}";

  # - Use "$mspPackageName+$mspConfigurationReleaseVerion" to find new 'install.sh' script in the new /opt/ directory.
  mspUpdateLocalFQN="${mspOptProviderDirectory}${mspPackageName}-${mspUpdateConfigFileReleaseVersion}/install.sh"

  # echo "mspUpdateLocalFQN=${mspUpdateLocalFQN}"

  if [ -f "${mspUpdateLocalFQN}" ] ;then
    if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Update handoff to new installion script, now."; fi

    # - Execute the new installation script, now.
    ${mspUpdateLocalFQN} ${mspVerboseOption}

    if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Update handoff to new installion script, has returned."; fi
  else
    if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Update script '${mspUpdateLocalFQN}' file not found."; fi
  fi
  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Update finished."; fi
fi

#-------------------------------------------------------------------------------
# Housekeeping and finish.

mspScriptExitNow
exit 0;  # 0 is 'Ok'.

################################################################################