#!/bin/bash
################################################################################
#
# Course Name:  Install MiniSAP on Your Personal Couputer
# Course URI:   https://www.installminisap.com/
# Script Owner: 'root' user in Virtual Machine guest operating system (OpenSUSE 15.1 Leap).
# Script Home:  /usr/local/bin/minisapplus.sh
# Description:  This is MiniSAP+, an everyday utility for limited access
#               user accounts to start/stop MiniSAP services, without having
#               to continually login as the '<sid>adm' user.
# Version:      0.4.2
# Valid For:    SAP Netweaver AS ABAP 7.50 SP02 Developer Edition (or above) for Linux.
# Author Name:  John Lang
# Author URI:   https://www.johnlang.com.au/
#
# - Exit return codes for this script:
#   - exit 0;  # 0 is 'Ok'.
#   - exit 1;  # 1 is 'Error - Invalid arguments passed to script.'.
#   - exit 2;  # 2 is 'Error - Power Off, Reboot or Autostart command failed.'.
#
# - Disclaimer: This script is a companion for the www.installminisap.com
#   course. It will work for the software combination specifically documented
#   in that course, for that version.
#
# - One Day: Add an execution log capability.
#
################################################################################
# [START EDITING] Add the password, version and user doco functions here.
################################################################################

mspOSMiniSAPAutostartDefault="yes"
mspOSMiniSAPAdminPasswordDefault="MiniSAP1"
mspOSMiniSAPAdminUserNameDefault=""
mspOSMiniSAPReleaseLifecycleDefault="stable"

mspFinishWithSleep="10"
mspFinishWithSleepForAutostart="30"
mspFinishWithSleepForReboot="5"
mspVersion="0.4.2"
mspProviderName="dataprofessionals.com.au"
mspPackageName="minisap-plus"



################################################################################
# [DO NOT EDIT] This next section of code is mostly a cookie template.
################################################################################

#-------------------------------------------------------------------------------
# Custom functions for execution of this scripts capability.

#
#
#

#-------------------------------------------------------------------------------
# Common functions for execution of this scripts core capabilities.

mspScriptUsage() { echo -n "
NAME
    ${mspScriptFilename} - MiniSAP Plus (+) for limited access user accounts.

SYNOPSIS
    ${mspScriptDirectory}${mspScriptFilename} [OPTION]...

DESCRIPTION
    Execute the 'MiniSAP Plus (+)' utilities for Gnome & OpenSUSE 15.x (Leap).

    A collection of quick start links that improve the MiniSAP Administrators
    experience while learning SAP technologies, in OpenSUSE 15.x (Leap).

    The 'MiniSAP Developer Edition' installations on your personal computer
    assume you wear many hats; student, developer, support, basis, etc.

    Your everyday experience will be through a normal (limited access) user
    account, like 'Demo'.

    However, each day you also need to login as the '<sid>ADM' user to execute
    'startsap.sh', 'stopsap.sh' and perform a range of other housekeeping
    activities. [Arrgghhhh]

    This '${mspScriptFilename}' script uses hardcoded variables to store the
    '<sid>adm' user login details. [Insecure and not for Production systems]

    Which you can then easily execute from a normal (limited access) user
    account (like 'Demo') using a terminal window or the Gnome desktop
    application action menu.

OPTIONS
        --autostart=<val> Set MiniSAP auto-start after login.
        --autostart-now   Execute minisapstart.sh, after Guest OS user login.
        --debug           Execute script in BASH debug mode. (set -x)
        --finish-key      Finish script execution when [Enter] key pressed.
        --finish-time     Finish script execution with timout (sleep ${mspFinishWithSleep}).
    -h, --help            Display this help and exit.
    -p, --password=<val>  Use <val> as password. (where required)
        --poweroff-now    Power off this machine, after minisapstop.sh.
    -q, --quiet           Execute quietly. Do not ask user anything.
        --reboot-now      Reboot this machine, after minisapstop.sh.
        --release=<val>   Use <val> 'Lifecycle' installation repository.
        --reset-now       Force User Name and Password to be reset to defaults.
    -s, --silent          Execute silently. Output nothing.
        --strict          Exit script on null variables. (set -o nounset)
    -u, --username=<val>  Use <val> as user name. (Like NPLADM & HXEADM)
    -v, --verbose         Output more information. (echo 'verbose')
        --version         Output version information and exit.

    Short options can be combined together. (-qv)

INSTRUCTIONS
    ${mspScriptDirectory}${mspScriptFilename} --help

    ${mspScriptDirectory}${mspScriptFilename} -p <password>
        Use this <password> instead. Ignore hardcoded value.

    ${mspScriptDirectory}${mspScriptFilename} -u <username> -p <password>
        Use this <username> and <password> instead. Ignore hardcoded value.

    ${mspScriptDirectory}${mspScriptFilename} --reboot-now
        Shutdown MiniSAP services then reboot the machine.

    ${mspScriptDirectory}${mspScriptFilename} --reset-now
        Force hardcoded User Name & Password to be reset to defaults.

    ${mspScriptDirectory}${mspScriptFilename} --reset-now --password=<password>
        Force hardcoded User Name to default and force Password to <password>.

    ${mspScriptDirectory}${mspScriptFilename} --release=<lifecycle>
        'stable' ..... for production release to general user community.
        'beta' ....... for feedback from the 'Courageous' user community.
        'alpha' ...... for prototyping with the 'Curious' user community.
        'unstable' ... for developers and unit testers.

    ${mspScriptDirectory}${mspScriptFilename} --autostart=<yes/no>
        'yes' ... to auto-start MiniSAP services after Guest OS user login.
        'no' .... to let Guest OS user manually start MiniSAP services.

EXIT STATUS CODES
    exit 0  Ok.
    exit 1  Error - Invalid arguments passed to script.
    exit 2  Error - Power Off, Reboot or Autostart command failed.

AUTHOR
    Written by John Thomas Lang.

REPORTING BUGS
    Support (Online): <https://www.installminisap.com/minisap-plus/support/>

COPYRIGHT
    (C) 2020 <https://www.installminisap.com/> (R) All rights reserved.
    License: Creative Commons Attribution-NonCommercial-ShareAlike v4.0.
    Online: CC BY-NC-SA 4.0 <http://creativecommons.org/licenses/by-nc-sa/4.0/>

SEE ALSO
    Manual (Online): <https://www.installminisap.com/minisap-plus/manual/>
    Manual (Local): /usr/local/share/com.installminisap.MiniSAPPlus.About.html

"
}

mspScriptExitNow() {
  if [ "${mspDoSilent}" == "0" ]; then
    echo "[MiniSAP+] Execution finished."

    # Useful when './minisapplus.sh' is executed within an existing installations
    # .desktop files 'Exec=' action, for user administration in text files.
    if [ "${mspDoFinishWithSleep}" != "0" ]; then
      echo -n -e "\n[MiniSAP+] Finish timeout set for '${mspFinishWithSleep}' seconds. "
      read -s -t ${mspFinishWithSleep}
      echo ''
    fi
    if [ "${mspDoFinishWithEnter}" != "0" ] && [ "${mspDoQuiet}" == "0" ]; then
      echo -n -e "\n[MiniSAP+] Press [Enter] or [Return] to Finish. "
      read -s
      echo ''
    fi
  fi
}

#-------------------------------------------------------------------------------

# Default options ... Do NOT edit.
mspDoAutostart=0
mspDoAutostartNow=0
mspDoConfigFileAutostart=0
mspDoConfigFilePassword=0
mspDoConfigFileReleaseLifecycle=0
mspDoConfigFileUserName=0
mspDoFinishWithEnter=0
mspDoFinishWithSleep=0
mspDoPassword=0
mspDoPowerOffNow=0
mspDoRebootNow=0
mspDoResetNow=0
mspDoHelp=0
mspDoQuiet=0
mspDoReleaseLifecycle=0
mspDoShellDebug=0
mspDoShellStrict=0
mspDoSilent=0
mspDoUserName=0
mspDoVerbose=0
mspDoVersion=0

# Default values ... Do NOT edit.
mspOSMiniSAPAutostart="";             # Always leave this one blank!
mspOSMiniSAPAutostartLine="";         # Always leave this one blank!
mspOSMiniSAPAdminPassword="";         # Always leave this one blank!
mspOSMiniSAPAdminPasswordLine="";     # Always leave this one blank!
mspOSMiniSAPAdminUserName="";         # Always leave this one blank!
mspOSMiniSAPAdminUserNameLine="";     # Always leave this one blank!
mspOSMiniSAPReleaseLifecycle="";      # Always leave this one blank!
mspOSMiniSAPReleaseLifecycleLine="";  # Always leave this one blank!
mspOSMiniSAPConfigurationAutostart="MiniSAP-Plus-Autostart-Services"
mspOSMiniSAPConfigurationOSAdminPassword="MiniSAP-OS-Admin-Password"
mspOSMiniSAPConfigurationOSAdminUserName="MiniSAP-OS-Admin-User-Name"
mspOSMiniSAPConfigurationReleaseLifecycle="MiniSAP-Plus-Release-Lifecycle"
mspResetPassword=""                   # Always leave this one blank!
mspResetUserName=""                   # Always leave this one blank!
mspScriptArgument=""
mspScriptArguments=()
mspScriptArgumentsInvalid=()
mspScriptArgumentsValid=()

# Defaults for installation update ... Do NOT edit.
mspUpdateReleaseLifecycleAlpha="alpha"
mspUpdateReleaseLifecycleBeta="beta"
mspUpdateReleaseLifecycleStable="stable"
mspUpdateReleaseLifecycleUnstable="unstable"

# Default configuration files ... Do NOT edit.
mspConfigFileAutostart=""
mspConfigFileOSAdminPassword=""
mspConfigFileOSAdminUserName=""
mspConfigFileReleaseLifecycle=""

# Default verbose output text ... Do Not Edit.
mspVerboseNewLine="\n"
mspVerboseOption="-v"

#-------------------------------------------------------------------------------
# Provider, Package, Directories and Files for default locations.
# Read Me: https://www.pathname.com/fhs/pub/fhs-2.3.html#OPTADDONAPPLICATIONSOFTWAREPACKAGES

# Default directories for storage and execution.
mspPackageVersionName="${mspPackageName}-${mspVersion}"
mspOptProviderDirectory="/opt/${mspProviderName}/"
mspOptPackageDirectory="/opt/${mspPackageName}/"
mspOptPackageLink="${mspOptPackageDirectory%\/}"
mspOptPackageVersionDirectory="/opt/${mspProviderName}/${mspPackageVersionName}/"
mspMiniSAPConfigurationLocalDirectory="/etc/${mspPackageName}/"
mspMiniSAPConfigurationLocalFQN="${mspMiniSAPConfigurationLocalDirectory}${mspPackageName}.conf"

# Capture and force current directory to be the location of this script.
mspScriptDirectory="$(dirname "$BASH_SOURCE")"
if [ "${mspScriptDirectory}" == "." ]; then
  mspScriptDirectory="$(pwd -P)";
fi
cd "${mspScriptDirectory}"
mspScriptDirectory="$(pwd -P)/"
mspScriptFilename="${0##*/}"

#-------------------------------------------------------------------------------
# Process the options and prepare for execution.

# Expand the compound options to be separated, for easier processing.
# Ensuring options start with '-', no outer quote wrapping.
# Forcing option values and arguments to get wrapped in outer quotes (').
mspScriptArguments=()
while (($#)); do
  case $1 in
    -[!-]?*) for ((i=1; i < ${#1}; i++)); do mspScriptArguments+=("-${1:i:1}"); done;;  # Expand '-abc' to '-a -b -c'.
    --?*=*)  mspScriptArguments+=("${1%%=*}" "'${1#*=}'");;  # Expand "--option={value}" to "--option 'value'".
    --?*)    mspScriptArguments+=("${1}");;
    --)      mspScriptArguments+=(--no-opt);;  # Dummy option for empty ones.
    *)       mspScriptArguments+=("${1@Q}");;
  esac
  shift
done
set -- "${mspScriptArguments[@]}";  # Put all expanded options back into official script array.

# Review each option, ignoring arguments and values ...
mspScriptArgumentsInvalid=()
mspScriptArgumentsValid=()
while (($#)); do
  mspScriptArgument="${1#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
  if [ "${mspScriptArgument:0:1}" == "-" ]; then
    if [ "${mspScriptArgument:0:2}" == "--" ]; then
      mspScriptArgument="${mspScriptArgument,,}";  # Make all lower case.
    fi
    case $mspScriptArgument in
         --autostart-now) mspDoAutostartNow=1;         mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --autostart)     mspDoAutostart=1;            mspScriptArgumentsValid+=("${mspScriptArgument}");
                          mspScriptArgument="${2#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
                          if [ "${mspScriptArgument:0:1}" != "-" ]; then
                            mspScriptArgument="${mspScriptArgument,,}";  # Make all lower case.
                            if [ "${mspScriptArgument}" != "" ]; then
                              if [ "${mspScriptArgument:0:1}" == "y" ]; then
                                  mspOSMiniSAPAutostart="yes";
                                  mspDoConfigFileAutostart=1;   mspScriptArgumentsValid+=("${mspScriptArgument}");
                              elif [ "${mspScriptArgument:0:1}" == "n" ]; then
                                  mspOSMiniSAPAutostart="no";
                                  mspDoConfigFileAutostart=1;   mspScriptArgumentsValid+=("${mspScriptArgument}");
                              else
                                mspScriptArgumentsInvalid+=("${mspScriptArgument@Q}");
                              fi
                            fi
                            shift
                          fi;;
         --debug)         mspDoShellDebug=1;           mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --finish-key)    mspDoFinishWithEnter=1;      mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --finish-time)   mspDoFinishWithSleep=1;      mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -h|--help)          mspDoHelp=1;                 mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -p|--password)      mspDoPassword=1;             mspScriptArgumentsValid+=("${mspScriptArgument}");
                          mspDoConfigFilePassword=1;
                          mspScriptArgument="${2#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
                          if [ "${mspScriptArgument:0:1}" != "-" ]; then
                            mspOSMiniSAPAdminPassword="${mspScriptArgument}";
                            mspResetPassword="${mspOSMiniSAPAdminPassword}";
                            mspScriptArgumentsValid+=("${mspScriptArgument}");
                            shift
                          fi;;
         --poweroff-now)  mspDoPowerOffNow=1;          mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -q|--quiet)         mspDoQuiet=1;                mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --reboot-now)    mspDoRebootNow=1;            mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --release)       mspDoReleaseLifecycle=1;     mspScriptArgumentsValid+=("${mspScriptArgument}");
                          mspDoConfigFileReleaseLifecycle=1;
                          mspScriptArgument="${2#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
                          if [ "${mspScriptArgument:0:1}" != "-" ]; then
                            mspScriptArgument="${mspScriptArgument,,}";  # Make all lower case.
                            if [ "${mspScriptArgument}" != "" ]; then
                              if [ "${mspScriptArgument}" == "${mspUpdateReleaseLifecycleAlpha}" ] ||
                                 [ "${mspScriptArgument}" == "${mspUpdateReleaseLifecycleBeta}" ] ||
                                 [ "${mspScriptArgument}" == "${mspUpdateReleaseLifecycleStable}" ] ||
                                 [ "${mspScriptArgument}" == "${mspUpdateReleaseLifecycleUnstable}" ]; then
                                  mspOSMiniSAPReleaseLifecycle="${mspScriptArgument}";
                                  mspScriptArgumentsValid+=("${mspScriptArgument}");
                              else
                                mspScriptArgumentsInvalid+=("${mspScriptArgument@Q}");
                              fi
                            fi
                            shift
                          fi;;
         --reset-now)     mspDoResetNow=1;              mspScriptArgumentsValid+=("${mspScriptArgument}");
                          mspDoConfigFilePassword=1;
                          mspDoConfigFileUserName=1;;
      -s|--silent)        mspDoSilent=1;                mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --strict)        mspDoShellStrict=1;           mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -u|--username)      mspDoUserName=1;              mspScriptArgumentsValid+=("${mspScriptArgument}");
                          mspDoConfigFileUserName=1;
                          mspScriptArgument="${2#\'}";  mspScriptArgument="${mspScriptArgument%\'}";
                          if [ "${mspScriptArgument:0:1}" != "-" ]; then
                            mspOSMiniSAPAdminUserName="${mspScriptArgument}";
                            mspResetUserName="${mspOSMiniSAPAdminUserName}";
                            mspScriptArgumentsValid+=("${mspScriptArgument}");
                            shift
                          fi;;
      -v|--verbose)       mspDoVerbose=1;               mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --version)       mspDoVersion=1;               mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --no-opt)        shift;;  # Ignore it.
      *)                  mspScriptArgumentsInvalid+=("${mspScriptArgument@Q}");;
    esac
  else
    mspScriptArgumentsValid+=("${mspScriptArgument}")
  fi
  shift
done
set -- "${mspScriptArgumentsValid[@]}";  # Put all valid arguments back into official script array.

# Process '--help' option before complaining about invalid options.
if [ "${mspDoHelp}" != 0 ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    mspScriptUsage
  fi
  mspScriptExitNow
  exit 0;  # 0 is 'Ok'.
fi

# Enforce use of only valid options.
if [ "${#mspScriptArgumentsInvalid[@]}" != 0 ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    echo "[MiniSAP+] Execution aborted."
    echo "[MiniSAP+] Invalid Options: ${mspScriptArgumentsInvalid[@]}"
    echo "[MiniSAP+] Use '${mspScriptDirectory}${mspScriptFilename} --help' option for full details."
  fi
  mspScriptExitNow
  exit 1;  # 1 is 'Error - Invalid arguments passed to script.'.
fi

#-------------------------------------------------------------------------------
# Set execution modes for shell.
# Reference URL: http://linuxcommand.org/lc3_man_pages/seth.html

if [ "${mspDoShellStrict}" != "0" ]; then
  set -o nounset
  if [ "$?" = 0 ] && [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Shell Execution: 'strict' mode enabled."
  fi
fi
set -o pipefail;  # Return the highest return code of piped commands.
# if [ "$?" = 0 ] && [ "${mspDoVerbose}" != "0" ]; then
#   echo "[MiniSAP+] Shell Execution: 'pipeline' mode enabled."
# fi
if [ "${mspDoShellDebug}" != "0" ]; then
  set -x
  if [ "$?" = 0 ] && [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Shell Execution: 'debug' mode enabled."
  fi
fi

#-------------------------------------------------------------------------------

if [ "${mspDoVerbose}" == "0" ] || [ "${mspDoSilent}" != "0" ]; then
  mspDoVerbose=0;  # Option '--silent' turns '--verbose' option off.
  mspVerboseNewLine=""
  mspVerboseOption=""
fi



################################################################################
# [START EDITING] Add the custom script code below this line.
################################################################################

#-------------------------------------------------------------------------------
# Process version options.

if [ "${mspDoVersion}" != "0" ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    echo "${mspVersion}";
  fi
  mspScriptExitNow
  exit 0;  # 0 is 'Ok'.
fi

#-------------------------------------------------------------------------------
# Process Power Off and Reboot options.

if [ "${mspDoPowerOffNow}" != "0" ]; then
  echo "[MiniSAP+] Power Off sequence starting ..."
  /usr/local/bin/minisapstop.sh
  mspFinishWithSleep="${mspFinishWithSleepForReboot}"
  mspScriptExitNow
  /sbin/shutdown -P now
  echo "[MiniSAP+] Error - Power Off sequence did not work properly."
  mspDoFinishWithEnter=1
  mspDoFinishWithSleep=0
  mspScriptExitNow
  exit 2;  # 2 is 'Error - Power Off, Reboot or Autostart command failed.'.
elif [ "${mspDoRebootNow}" != "0" ]; then
  echo "[MiniSAP+] Reboot sequence starting ..."
  /usr/local/bin/minisapstop.sh
  mspFinishWithSleep="${mspFinishWithSleepForReboot}"
  mspScriptExitNow
  /sbin/reboot
  echo "[MiniSAP+] Error - Power Off sequence did not work properly."
  mspDoFinishWithEnter=1
  mspDoFinishWithSleep=0
  mspScriptExitNow
  exit 2;  # 2 is 'Error - Power Off, Reboot or Autostart command failed.'.
fi

#-------------------------------------------------------------------------------
# Execution healthcheck.

if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Execution healthcheck starting ..."; fi

# Check for which password should be used.
if [ "${mspDoResetNow}" != "0" ]; then
  mspDoPassword=1
  mspDoConfigFilePassword=1
  mspOSMiniSAPAdminPassword="${mspResetPassword}"
  if [ "${mspOSMiniSAPAdminPassword}" == "" ]; then
    mspOSMiniSAPAdminPassword="${mspOSMiniSAPAdminPasswordDefault}"
  fi
fi
mspConfigFileOSAdminPassword=""
mspConfigFileOSAdminPasswordLine="$(grep --no-messages -i -e "^${mspOSMiniSAPConfigurationOSAdminPassword}=.*" ${mspMiniSAPConfigurationLocalFQN})"
if [ "${mspConfigFileOSAdminPasswordLine}" != "" ]; then
  mspConfigFileOSAdminPasswordLine="${mspConfigFileOSAdminPasswordLine//\"/\'}";  # Replace all " with '.
  mspConfigFileOSAdminPassword="${mspConfigFileOSAdminPasswordLine#*=}"
  mspConfigFileOSAdminPassword="${mspConfigFileOSAdminPassword#\'}"; mspConfigFileOSAdminPassword="${mspConfigFileOSAdminPassword%\'}";
fi
if [ "${mspOSMiniSAPAdminPassword}" == "" ]; then
  mspOSMiniSAPAdminPassword="${mspOSMiniSAPAdminPasswordDefault}"
  if [ "${mspConfigFileOSAdminPassword}" != "" ] && [ "${mspDoResetNow}" == "0" ]; then
    mspOSMiniSAPAdminPassword="${mspConfigFileOSAdminPassword}"
  fi
fi
mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPConfigurationOSAdminPassword}=\"${mspOSMiniSAPAdminPassword}\""
mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPAdminPasswordLine//\'/\\\'}";  # Escape ' for sed execution.

# Check for which user name should be used.
if [ "${mspDoResetNow}" != "0" ]; then
  mspDoUserName=1
  mspDoConfigFileUserName=1
  mspOSMiniSAPAdminUserName="${mspResetUserName}"
  if [ "${mspOSMiniSAPAdminUserName}" == "" ]; then
    mspOSMiniSAPAdminUserName="${mspOSMiniSAPAdminUserNameDefault}"
  fi
fi
mspConfigFileOSAdminUserName=""
mspConfigFileOSAdminUserNameLine="$(grep --no-messages -i -e "^${mspOSMiniSAPConfigurationOSAdminUserName}=.*" ${mspMiniSAPConfigurationLocalFQN})"
if [ "${mspConfigFileOSAdminUserNameLine}" != "" ]; then
  mspConfigFileOSAdminUserNameLine="${mspConfigFileOSAdminUserNameLine//\"/\'}";  # Replace all " with '.
  mspConfigFileOSAdminUserName="${mspConfigFileOSAdminUserNameLine#*=}"
  mspConfigFileOSAdminUserName="${mspConfigFileOSAdminUserName#\'}"; mspConfigFileOSAdminUserName="${mspConfigFileOSAdminUserName%\'}";
fi
if [ "${mspOSMiniSAPAdminUserName}" == "" ]; then
  mspOSMiniSAPAdminUserName="${mspOSMiniSAPAdminUserNameDefault}"
  if [ "${mspConfigFileOSAdminUserName}" != "" ] && [ "${mspDoResetNow}" == "0" ]; then
    mspOSMiniSAPAdminUserName="${mspConfigFileOSAdminUserName}"
  fi
fi
mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPConfigurationOSAdminUserName}=\"${mspOSMiniSAPAdminUserName}\""
mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPAdminUserNameLine//\'/\\\'}";  # Escape ' for sed execution.

# Get Configuration - MiniSAP Release Lifecycle to be used.
mspConfigFileReleaseLifecycle=""
mspOSMiniSAPReleaseLifecycleLine="$(grep --no-messages -i -e "^${mspOSMiniSAPConfigurationReleaseLifecycle}=.*" ${mspMiniSAPConfigurationLocalFQN})"
if [ "${mspOSMiniSAPReleaseLifecycleLine}" != "" ]; then
  mspOSMiniSAPReleaseLifecycleLine="${mspOSMiniSAPReleaseLifecycleLine//\"/\'}";  # Replace all " with '.
  mspConfigFileReleaseLifecycle="${mspOSMiniSAPReleaseLifecycleLine#*=}"
  mspConfigFileReleaseLifecycle="${mspConfigFileReleaseLifecycle#\'}"; mspConfigFileReleaseLifecycle="${mspConfigFileReleaseLifecycle%\'}";
fi
if [ "${mspOSMiniSAPReleaseLifecycle}" == "" ]; then
  mspOSMiniSAPReleaseLifecycle="${mspConfigFileReleaseLifecycle}"
  if [ "${mspOSMiniSAPReleaseLifecycle}" == "" ]; then
    mspOSMiniSAPReleaseLifecycle="${mspOSMiniSAPReleaseLifecycleDefault}"
  fi
fi
mspOSMiniSAPReleaseLifecycle="${mspOSMiniSAPReleaseLifecycle,,}";  # Make all lower case.
mspOSMiniSAPReleaseLifecycleLine="${mspOSMiniSAPConfigurationReleaseLifecycle}=\"${mspOSMiniSAPReleaseLifecycle}\""
mspOSMiniSAPReleaseLifecycleLine="${mspOSMiniSAPReleaseLifecycleLine//\'/\\\'}";  # Escape ' for sed execution.
if [ "${mspOSMiniSAPReleaseLifecycle}" != "" ]; then
  mspDoConfigFileReleaseLifecycle=1
fi

# Get Configuration - MiniSAP Auto-start Services to be used.
mspConfigFileAutostart=""
mspOSMiniSAPAutostartLine="$(grep --no-messages -i -e "^${mspOSMiniSAPConfigurationAutostart}=.*" ${mspMiniSAPConfigurationLocalFQN})"
if [ "${mspOSMiniSAPAutostartLine}" != "" ]; then
  mspOSMiniSAPAutostartLine="${mspOSMiniSAPAutostartLine//\"/\'}";  # Replace all " with '.
  mspConfigFileAutostart="${mspOSMiniSAPAutostartLine#*=}"
  mspConfigFileAutostart="${mspConfigFileAutostart#\'}"; mspConfigFileAutostart="${mspConfigFileAutostart%\'}";
fi
if [ "${mspOSMiniSAPAutostart}" == "" ]; then
  mspOSMiniSAPAutostart="${mspConfigFileAutostart}"
  if [ "${mspOSMiniSAPAutostart}" == "" ]; then
    mspOSMiniSAPAutostart="${mspOSMiniSAPAutostartDefault}"
  fi
fi
mspOSMiniSAPAutostart="${mspOSMiniSAPAutostart,,}";  # Make all lower case.
mspOSMiniSAPAutostartLine="${mspOSMiniSAPConfigurationAutostart}=\"${mspOSMiniSAPAutostart}\""
mspOSMiniSAPAutostartLine="${mspOSMiniSAPAutostartLine//\'/\\\'}";  # Escape ' for sed execution.
if [ "${mspOSMiniSAPAutostart}" != "" ]; then
  mspDoConfigFileAutostart=1
fi

if [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] Execution Directory .... : ${mspScriptDirectory}"
  echo "[MiniSAP+] Execution Filename ..... : ${mspScriptFilename}"
  echo "[MiniSAP+] Config Autostart Services: ${mspOSMiniSAPAutostart}"
  echo "[MiniSAP+] Config OS Admin User Name: ${mspOSMiniSAPAdminUserName}"
  echo "[MiniSAP+] Config OS Admin Password : ${mspOSMiniSAPAdminPassword}"
  echo "[MiniSAP+] Config Release Lifecycle : ${mspOSMiniSAPReleaseLifecycle}"
  echo "[MiniSAP+] Reset Option ........... : ${mspDoResetNow}"
fi

if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Execution healthcheck finished."; fi

#-------------------------------------------------------------------------------
# Lets do it ... assuming use of Gnome desktop in OpenSUSE 15 (Leap).
# ... because this script is a companion for <https://www.installminisap.com/>.

if [ "${mspDoSilent}" == "0" ]; then echo "[MiniSAP+] Execution starting ..."; fi

#-------------------------------------------------------------------------------
# Process Autostart Services option.

if [ "${mspDoAutostartNow}" != "0" ]; then
  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Autostart Services sequence starting ..."; fi
  if [ "${mspOSMiniSAPAutostart:0:1}" == "y" ]; then
    /usr/local/bin/minisapstart.sh
  else
    echo "[MiniSAP+] Autostart Services sequence ... skipping 'minisapstart.sh'."
  fi



  if [ "${mspDoVerbose}" != "0" ]; then echo "[MiniSAP+] Autostart Services sequence finished."; fi
  mspFinishWithSleep="${mspFinishWithSleepForAutostart}"
  mspScriptExitNow
  exit 0;  # 0 is 'Ok'.
elif [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] Autostart Services sequence ... will be skipped."
fi

#-------------------------------------------------------------------------------
# Ask user to confirm the 'Autostart Services' answer they want as their default.

if [ "${mspDoAutostart}" != "0" ]; then
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Autostart Services choice starting ..."
  fi
  echo "[MiniSAP+] Currently "${mspOSMiniSAPAutostartLine}"."
  if [ "${mspDoQuiet}" == "0" ]; then
    echo "Should MiniSAP+ try to auto-start services after Guest OS user login?"
    echo "    (y)es ... to auto-start MiniSAP services after Guest OS user login."
    echo "    (n)o .... to let Guest OS user manually start the MiniSAP services."
    read -e -i "${mspOSMiniSAPAutostart}" -p "Choice [y/n]: "
    mspScriptArgument="${REPLY,,}";  # Make all lower case.
    case "${mspScriptArgument:0:1}" in
      y) mspOSMiniSAPAutostart="yes";;
      n) mspOSMiniSAPAutostart="no";;
    esac
  fi
  if [ "${mspOSMiniSAPAutostart}" == "" ]; then
    mspOSMiniSAPAutostart="${mspOSMiniSAPAutostartDefault}"
  fi
  mspOSMiniSAPAutostart="${mspOSMiniSAPAutostart,,}";  # Make all lower case.
  mspOSMiniSAPAutostartLine="${mspOSMiniSAPConfigurationAutostart}=\"${mspOSMiniSAPAutostart}\""
  mspOSMiniSAPAutostartLine="${mspOSMiniSAPAutostartLine//\'/\\\'}";  # Escape ' for sed execution.
  mspDoConfigFileAutostart=1
  echo "[MiniSAP+] Setting "${mspOSMiniSAPAutostartLine}"."
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Autostart Services choice finished."
  fi
elif [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] Autostart Services choice ... will be skipped."
fi

#-------------------------------------------------------------------------------
# Ask user to confirm the 'Release Lifecycle' they want as their default.

if [ "${mspDoReleaseLifecycle}" != "0" ]; then
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Release Lifecycle choice starting ..."
  fi
  echo "[MiniSAP+] Currently "${mspOSMiniSAPReleaseLifecycleLine}"."
  if [ "${mspDoQuiet}" == "0" ]; then
    echo "Which 'Release Lifecycle' would you like the default update option to be?"
    echo "    (s)table ..... for production release to general user community."
    echo "    (b)eta ....... for feedback from the 'Courageous' user community."
    echo "    (a)lpha ...... for prototyping with the 'Curious' user community."
    echo "    (u)nstable ... for developers and unit testers."
    read -e -i "${mspOSMiniSAPReleaseLifecycle}" -p "Choice [s/b/a/u]: "
    mspScriptArgument="${REPLY,,}";  # Make all lower case.
    case "${mspScriptArgument:0:1}" in
      "${mspUpdateReleaseLifecycleAlpha:0:1}")    mspOSMiniSAPReleaseLifecycle="${mspUpdateReleaseLifecycleAlpha}";;
      "${mspUpdateReleaseLifecycleBeta:0:1}")     mspOSMiniSAPReleaseLifecycle="${mspUpdateReleaseLifecycleBeta}";;
      "${mspUpdateReleaseLifecycleStable:0:1}")   mspOSMiniSAPReleaseLifecycle="${mspUpdateReleaseLifecycleStable}";;
      "${mspUpdateReleaseLifecycleUnstable:0:1}") mspOSMiniSAPReleaseLifecycle="${mspUpdateReleaseLifecycleUnstable}";;
    esac
  fi
  if [ "${mspOSMiniSAPReleaseLifecycle}" == "" ]; then
    mspOSMiniSAPReleaseLifecycle="${mspOSMiniSAPReleaseLifecycleDefault}"
  fi
  mspOSMiniSAPReleaseLifecycle="${mspOSMiniSAPReleaseLifecycle,,}";  # Make all lower case.
  mspOSMiniSAPReleaseLifecycleLine="${mspOSMiniSAPConfigurationReleaseLifecycle}=\"${mspOSMiniSAPReleaseLifecycle}\""
  mspOSMiniSAPReleaseLifecycleLine="${mspOSMiniSAPReleaseLifecycleLine//\'/\\\'}";  # Escape ' for sed execution.
  mspDoConfigFileReleaseLifecycle=1
  echo "[MiniSAP+] Setting "${mspOSMiniSAPReleaseLifecycleLine}"."
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Release Lifecycle choice finished."
  fi
elif [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] Release Lifecycle choice ... will be skipped."
fi

#-------------------------------------------------------------------------------
# Ask user to confirm the 'OS Admin Username' they want as their default.

if [ "${mspDoUserName}" != "0" ]; then
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] OS Admin User Name entry starting ..."
  fi
  if [ "${mspConfigFileOSAdminUserNameLine}" != "" ] && [ "${mspDoResetNow}" != "0" ]; then
    echo "[MiniSAP+] Currently "${mspConfigFileOSAdminUserNameLine}"."
  else
    echo "[MiniSAP+] Currently "${mspOSMiniSAPAdminUserNameLine}"."
  fi
  if [ "${mspDoQuiet}" == "0" ]; then
    echo "Which 'Guest OS Admin User Name' should start/stop the MiniSAP services?"
    read -e -i "${mspOSMiniSAPAdminUserName}" -p "OS Admin User Name: "
    mspOSMiniSAPAdminUserName="${REPLY}";
  fi
  if [ "${mspOSMiniSAPAdminUserName}" == "" ]; then
    mspOSMiniSAPAdminUserName="${mspOSMiniSAPAdminUserNameDefault}"
  fi
  mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPConfigurationOSAdminUserName}=\"${mspOSMiniSAPAdminUserName}\""
  mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPAdminUserNameLine//\'/\\\'}";  # Escape ' for sed execution.
  mspDoConfigFileUserName=1
  echo "[MiniSAP+] Setting "${mspOSMiniSAPAdminUserNameLine}"."
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] OS Admin User Name entry finished."
  fi
elif [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] OS Admin User Name entry ... will be skipped."
fi

#-------------------------------------------------------------------------------
# Ask user to confirm the 'OS Admin Password' they want as their default.

if [ "${mspDoPassword}" != "0" ]; then
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] OS Admin Password entry starting ..."
  fi
  if [ "${mspConfigFileOSAdminPasswordLine}" != "" ] && [ "${mspDoResetNow}" != "0" ]; then
    echo "[MiniSAP+] Currently "${mspConfigFileOSAdminPasswordLine}"."
  else
    echo "[MiniSAP+] Currently "${mspOSMiniSAPAdminPasswordLine}"."
  fi
  if [ "${mspDoQuiet}" == "0" ]; then
    echo "Which 'Guest OS Admin Password' will allow start/stop of MiniSAP services?"
    read -e -i "${mspOSMiniSAPAdminPassword}" -p "OS Admin Password: "
    mspOSMiniSAPAdminPassword="${REPLY}";
  fi
  if [ "${mspOSMiniSAPAdminPassword}" == "" ]; then
    mspOSMiniSAPAdminPassword="${mspOSMiniSAPAdminPasswordDefault}"
  fi
  mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPConfigurationOSAdminPassword}=\"${mspOSMiniSAPAdminPassword}\""
  mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPAdminPasswordLine//\'/\\\'}";  # Escape ' for sed execution.
  mspDoConfigFilePassword=1
  echo "[MiniSAP+] Setting "${mspOSMiniSAPAdminPasswordLine}"."
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] OS Admin Password entry finished."
  fi
elif [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] OS Admin Password entry ... will be skipped."
fi

#-------------------------------------------------------------------------------
# Update '/etc/minisap-plus/minisap-plus.conf' scripts to edit the default MiniSAP Admin.

# Update 'MiniSAP-Plus-Autostart-Services=' variable defeult value in target scripts.
if [ "${mspDoConfigFileAutostart}" != "0" ]; then
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Autostart Services update starting ..."
  fi
  grep --silent --no-messages -i -e "^${mspOSMiniSAPConfigurationAutostart}=.*" ${mspMiniSAPConfigurationLocalFQN} && sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationAutostart}=.*/${mspOSMiniSAPAutostartLine}/I" ${mspMiniSAPConfigurationLocalFQN} || echo -e "\n${mspOSMiniSAPAutostartLine}" >> ${mspMiniSAPConfigurationLocalFQN}
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Autostart Services update finished."
  fi
elif [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] Config - Autostart Services update ... will be skipped."
fi

# Update 'MiniSAP-OS-Admin-User-Name=' variable defeult value in target scripts.
if [ "${mspDoConfigFileUserName}" != "0" ]; then
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Admin User Name update starting ..."
  fi
  # echo -n "$(sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationOSAdminUserName}=.*/${mspOSMiniSAPAdminUserNameLine}/I" ${mspMiniSAPConfigurationLocalFQN})"; echo -ne "${mspVerboseNewLine}";
  grep --silent --no-messages -i -e "^${mspOSMiniSAPConfigurationOSAdminUserName}=.*" ${mspMiniSAPConfigurationLocalFQN} && sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationOSAdminUserName}=.*/${mspOSMiniSAPAdminUserNameLine}/I" ${mspMiniSAPConfigurationLocalFQN} || echo -e "\n${mspOSMiniSAPAdminUserNameLine}" >> ${mspMiniSAPConfigurationLocalFQN}
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Admin User Name update finished."
  fi
elif [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] Config - Admin User Name update ... will be skipped."
fi

# Update 'MiniSAP-OS-Admin-Password=' variable defeult value in target scripts.
if [ "${mspDoConfigFilePassword}" != "0" ]; then
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Admin Password update starting ..."
  fi
  # echo -n "$(sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationOSAdminPassword}=.*/${mspOSMiniSAPAdminPasswordLine}/I" ${mspMiniSAPConfigurationLocalFQN})"; echo -ne "${mspVerboseNewLine}";
  grep --silent --no-messages -i -e "^${mspOSMiniSAPConfigurationOSAdminPassword}=.*" ${mspMiniSAPConfigurationLocalFQN} && sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationOSAdminPassword}=.*/${mspOSMiniSAPAdminPasswordLine}/I" ${mspMiniSAPConfigurationLocalFQN} || echo -e "\n${mspOSMiniSAPAdminPasswordLine}" >> ${mspMiniSAPConfigurationLocalFQN}
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Admin Password update finished."
  fi
elif [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] Config - Admin Password update ... will be skipped."
fi

# Update 'MiniSAP-Plus-Release-Lifecycle=' variable defeult value in target scripts.
if [ "${mspDoConfigFileReleaseLifecycle}" != "0" ]; then
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Release Lifecycle update starting ..."
  fi
  # echo -n "$(sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationReleaseLifecycle}=.*/${mspOSMiniSAPReleaseLifecycleLine}/I" ${mspMiniSAPConfigurationLocalFQN})"; echo -ne "${mspVerboseNewLine}";
  grep --silent --no-messages -i -e "^${mspOSMiniSAPConfigurationReleaseLifecycle}=.*" ${mspMiniSAPConfigurationLocalFQN} && sed --in-place --separate --unbuffered "s/^${mspOSMiniSAPConfigurationReleaseLifecycle}=.*/${mspOSMiniSAPReleaseLifecycleLine}/I" ${mspMiniSAPConfigurationLocalFQN} || echo -e "\n${mspOSMiniSAPReleaseLifecycleLine}" >> ${mspMiniSAPConfigurationLocalFQN}
  if [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Config - Release Lifecycle update finished."
  fi
elif [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] Config - Release Lifecycle update ... will be skipped."
fi

#-------------------------------------------------------------------------------
# Housekeeping and finish.

mspScriptExitNow
exit 0;  # 0 is 'Ok'.

################################################################################