#!/bin/bash
################################################################################
#
# Course Name:  Install MiniSAP on Your Personal Couputer
# Course URI:   https://www.installminisap.com/
# Script Owner: 'root' user in Virtual Machine guest operating system (OpenSUSE 15.1 Leap).
# Script Home:  /usr/local/bin/minisapstart.sh
# Description:  Easily stop MiniSAP services from a limited access user
#               account, without having to continually login as <sid>ADM user.
# Version:      0.4.1
# Valid For:    SAP Netweaver AS ABAP 7.50 SP02 Developer Edition (or above) for Linux.
# Author Name:  John Lang
# Author URI:   https://www.johnlang.com.au/
#
# - Exit return codes for this script:
#   - exit 0;  # 0 is 'Ok'.
#   - exit 1;  # 1 is 'Error - Invalid arguments passed to script.'.
#   - exit 2;  # 2 is 'Error - Unable to determine username and password to use.'.
#
# - Disclaimer: This script is a companion for the www.installminisap.com
#   course. It will work for the software combination specifically documented
#   in that course, for that version.
#
# - One Day: Add an execution log capability.
#
################################################################################
# [START EDITING] Add the password, version and user doco functions here.
################################################################################

mspOSMiniSAPAdminPasswordDefault="MiniSAP1"
mspFinishWithSleep="30"
mspVersion="0.4.1"
mspProviderName="dataprofessionals.com.au"
mspPackageName="minisap-plus"



################################################################################
# [DO NOT EDIT] This next section of code is mostly a cookie template.
################################################################################

#-------------------------------------------------------------------------------
# Custom functions for execution of this scripts capability.

#
#
#

#-------------------------------------------------------------------------------
# Common functions for execution of this scripts core capabilities.

mspScriptUsage() { echo -n "
NAME
    ${mspScriptFilename} - Stop MiniSAP services from limited access user account.

SYNOPSIS
    ${mspScriptDirectory}${mspScriptFilename} [OPTION]...

DESCRIPTION
    Stop the MiniSAP services in this system, so you can shutdown the virtual
    machine and go enjoy some sunshine because the birds are being noisy now.

    The 'MiniSAP Developer Edition' installations on your personal computer
    assume you wear many hats; student, developer, support, basis, etc.

    Your everyday experience will be through a normal (limited access) user
    account, like 'Demo'.

    However, each day you also need to login as the '<sid>ADM' user to execute
    'startsap.sh', 'stopsap.sh' and perform a range of other housekeeping
    activities. [Arrgghhhh]

    This '${mspScriptFilename}' script uses hardcoded variables to store the
    '<sid>adm' user login details. [Insecure and not for Production systems]

    Which you can then easily execute from a normal (limited access) user
    account (like 'Demo') using a terminal window or the Gnome desktop
    application action menu.

OPTIONS
        --debug           Execute script in BASH debug mode. (set -x)
        --finish-key      Finish script execution when [Enter] key pressed.
        --finish-time     Finish script execution with timout (sleep ${mspFinishWithSleep}).
    -h, --help            Display this help and exit.
    -p, --password=<val>  Use <val> as password. (where required)
    -q, --quiet           Execute quietly. Do not ask user anything.
    -s, --silent          Execute silently. Output nothing.
        --strict          Exit script on null variables. (set -o nounset)
    -u, --username=<val>  Use <val> as user name. (Like NPLADM & HXEADM)
    -v, --verbose         Output more information. (echo 'verbose')
        --version         Output version information and exit.

    Short options can be combined together. (-qv)

INSTRUCTIONS
    ${mspScriptDirectory}${mspScriptFilename} --help

    ${mspScriptDirectory}${mspScriptFilename} -p <password>
        Use this <password> instead. Ignore hardcoded value.

    ${mspScriptDirectory}${mspScriptFilename} -u <username> -p <password>
        Use this <username> and <password> instead. Ignore hardcoded value.

    ${mspScriptDirectory}${mspScriptFilename} --verbose --finish-time
        Verbose output and wait for '${mspFinishWithSleep}' before exiting.

EXIT STATUS CODES
    exit 0  Ok.
    exit 1  Error - Invalid arguments passed to script.
    exit 2  Error - Unable to determine username and password to use.

AUTHOR
    Written by John Thomas Lang.

REPORTING BUGS
    Support (Online): <https://www.installminisap.com/minisap-plus/support/>

COPYRIGHT
    (C) 2020 <https://www.installminisap.com/> (R) All rights reserved.
    License: Creative Commons Attribution-NonCommercial-ShareAlike v4.0.
    Online: CC BY-NC-SA 4.0 <http://creativecommons.org/licenses/by-nc-sa/4.0/>

SEE ALSO
    Manual (Online): <https://www.installminisap.com/minisap-plus/manual/>
    Manual (Local): /usr/local/share/com.installminisap.MiniSAPPlus.About.html

"
}

mspScriptExitNow() {
  if [ "${mspDoSilent}" == "0" ]; then

    # Useful when './minisapstop.sh' is executed within existing installations
    # .desktop files 'Exec=' action, for a 'MiniSAP Plus (+)' upgrade.
    if [ "${mspDoFinishWithSleep}" != "0" ]; then
      echo -n -e "\n[MiniSAP+] Finish timeout set for '${mspFinishWithSleep}' seconds. "
      read -s -t ${mspFinishWithSleep}
      echo ''
    fi
    if [ "${mspDoFinishWithEnter}" != "0" ] && [ "${mspDoQuiet}" == "0" ]; then
      echo -n -e "\n[MiniSAP+] Press [Enter] or [Return] to Finish. "
      read -s
      echo ''
    fi
  fi
}

#-------------------------------------------------------------------------------

# Default options ... Do NOT edit.
mspDoFinishWithEnter=0
mspDoFinishWithSleep=0
mspDoHelp=0
mspDoPassword=0
mspDoQuiet=0
mspDoShellDebug=0
mspDoShellStrict=0
mspDoSilent=0
mspDoUserName=0
mspDoVerbose=0
mspDoVersion=0

# Default values ... Do NOT edit.
mspOSMiniSAPAdminPassword="";        # Always leave this one blank!
mspOSMiniSAPAdminPasswordLine="";    # Always leave this one blank!
mspOSMiniSAPAdminUserName="";        # Always leave this one blank!
mspOSMiniSAPAdminUserNameDefault=""  # Always leave this one blank!
mspOSMiniSAPAdminUserNameLine="";    # Always leave this one blank!
mspOSMiniSAPConfigurationOSAdminUserName="MiniSAP-OS-Admin-User-Name"
mspOSMiniSAPConfigurationOSAdminPassword="MiniSAP-OS-Admin-Password"
mspScriptArgument=""
mspScriptArguments=()
mspScriptArgumentsInvalid=()
mspScriptArgumentsValid=()
#mspVerboseNewLine="\n"
#mspVerboseOption="-v"

#-------------------------------------------------------------------------------
# Provider, Package, Directories and Files for default locations.
# Read Me: https://www.pathname.com/fhs/pub/fhs-2.3.html#OPTADDONAPPLICATIONSOFTWAREPACKAGES

# Default directories for storage and execution.
mspPackageVersionName="${mspPackageName}-${mspVersion}"
mspOptProviderDirectory="/opt/${mspProviderName}/"
mspOptPackageDirectory="/opt/${mspPackageName}/"
mspOptPackageLink="${mspOptPackageDirectory%\/}"
mspOptPackageVersionDirectory="/opt/${mspProviderName}/${mspPackageVersionName}/"
mspMiniSAPConfigurationLocalFQN="/etc/${mspPackageName}/${mspPackageName}.conf"

# Capture and force current directory to be the location of this script.
mspScriptDirectory="$(dirname "$BASH_SOURCE")"
if [ "${mspScriptDirectory}" == "." ]; then
  mspScriptDirectory="$(pwd -P)";
fi
cd "${mspScriptDirectory}"
mspScriptDirectory="$(pwd -P)/"
mspScriptFilename="${0##*/}"

#-------------------------------------------------------------------------------
# Process the options and prepare for execution.

# Expand the compound options to be separated, for easier processing.
# Ensuring options start with '-', no outer quote wrapping.
# Forcing option values and arguments to get wrapped in outer quotes (').
mspScriptArguments=()
while (($#)); do
  case $1 in
    -[!-]?*) for ((i=1; i < ${#1}; i++)); do mspScriptArguments+=("-${1:i:1}"); done;;  # Expand '-abc' to '-a -b -c'.
    --?*=*)  mspScriptArguments+=("${1%%=*}" "'${1#*=}'");;  # Expand "--option={value}" to "--option 'value'".
    --?*)    mspScriptArguments+=("${1}");;
    --)      mspScriptArguments+=(--no-opt);;  # Dummy option for empty ones.
    *)       mspScriptArguments+=("${1@Q}");;
  esac
  shift
done
set -- "${mspScriptArguments[@]}";  # Put all expanded options back into official script array.

# Review each option, ignoring arguments and values ...
mspScriptArgumentsInvalid=()
mspScriptArgumentsValid=()
while (($#)); do
  mspScriptArgument="${1#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
  if [ "${mspScriptArgument:0:1}" == "-" ]; then
    if [ "${mspScriptArgument:0:2}" == "--" ]; then
      mspScriptArgument="${mspScriptArgument,,}";  # Make all lower case.
    fi
    case $mspScriptArgument in
         --debug)       mspDoShellDebug=1;      mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --finish-key)  mspDoFinishWithEnter=1; mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --finish-time) mspDoFinishWithSleep=1; mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -h|--help)        mspDoHelp=1;            mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -p|--password)    mspDoPassword=1;        mspScriptArgumentsValid+=("${mspScriptArgument}");
                        mspScriptArgument="${2#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
                        if [ "${mspScriptArgument:0:1}" != "-" ]; then
                          mspOSMiniSAPAdminPassword="${mspScriptArgument}";
                          mspScriptArgumentsValid+=("${mspScriptArgument}");
                          shift;
                        fi;;
      -q|--quiet)       mspDoQuiet=1;           mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -s|--silent)      mspDoSilent=1;          mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --strict)      mspDoShellStrict=1;     mspScriptArgumentsValid+=("${mspScriptArgument}");;
      -u|--username)    mspDoUserName=1;        mspScriptArgumentsValid+=("${mspScriptArgument}");
                        mspScriptArgument="${2#\'}"; mspScriptArgument="${mspScriptArgument%\'}";
                        if [ "${mspScriptArgument:0:1}" != "-" ]; then
                          mspOSMiniSAPAdminUserName="${mspScriptArgument}";
                          mspScriptArgumentsValid+=("${mspScriptArgument}");
                          shift;
                        fi;;
      -v|--verbose)     mspDoVerbose=1;         mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --version)     mspDoVersion=1;         mspScriptArgumentsValid+=("${mspScriptArgument}");;
         --no-opt)      shift;;  # Ignore it.
      *)                mspScriptArgumentsInvalid+=("${mspScriptArgument@Q}");;
    esac
  else
    mspScriptArgumentsValid+=("${mspScriptArgument}")
  fi
  shift
done
set -- "${mspScriptArgumentsValid[@]}";  # Put all valid arguments back into official script array.

# Process '--help' option before complaining about invalid options.
if [ "${mspDoHelp}" != 0 ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    mspScriptUsage
  fi
  mspScriptExitNow
  exit 0;  # 0 is 'Ok'.
fi

# Enforce use of only valid options.
if [ "${#mspScriptArgumentsInvalid[@]}" != 0 ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    echo "[MiniSAP+] Execution aborted."
    echo "[MiniSAP+] Invalid Options: ${mspScriptArgumentsInvalid[@]}"
    echo "[MiniSAP+] Use '${mspScriptDirectory}${mspScriptFilename} --help' option for full details."
  fi
  mspScriptExitNow
  exit 1;  # 1 is 'Error - Invalid arguments passed to script.'.
fi

#-------------------------------------------------------------------------------
# Set execution modes for shell.
# Reference URL: http://linuxcommand.org/lc3_man_pages/seth.html

if [ "${mspDoShellStrict}" != "0" ]; then
  set -o nounset
  if [ "$?" = 0 ] && [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Shell Execution: 'strict' mode enabled."
  fi
fi
set -o pipefail;  # Return the highest return code of piped commands.
# if [ "$?" = 0 ] && [ "${mspDoVerbose}" != "0" ]; then
#   echo "[MiniSAP+] Shell Execution: 'pipeline' mode enabled."
# fi
if [ "${mspDoShellDebug}" != "0" ]; then
  set -x
  if [ "$?" = 0 ] && [ "${mspDoVerbose}" != "0" ]; then
    echo "[MiniSAP+] Shell Execution: 'debug' mode enabled."
  fi
fi

#-------------------------------------------------------------------------------

if [ "${mspDoVerbose}" == "0" ] || [ "${mspDoSilent}" != "0" ]; then
  mspDoVerbose=0;  # Option '--silent' turns '--verbose' option off.
#  mspVerboseNewLine=""
#  mspVerboseOption=""
fi



################################################################################
# [START EDITING] Add the custom script code below this line.
################################################################################

#-------------------------------------------------------------------------------
# Process version options.

if [ "${mspDoVersion}" != "0" ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    echo "${mspVersion}";
  fi
  mspScriptExitNow
  exit 0;  # 0 is 'Ok'.
fi

#-------------------------------------------------------------------------------
# Installation healthcheck.

# Check for password to be used.
mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPConfigurationOSAdminPassword}='${mspOSMiniSAPAdminPassword}'"
if [ "${mspOSMiniSAPAdminPassword}" == "" ]; then
  mspOSMiniSAPAdminPasswordLine="$(grep --no-messages -i -e "^${mspOSMiniSAPConfigurationOSAdminPassword}=.*" ${mspMiniSAPConfigurationLocalFQN})"
  if [ "${mspOSMiniSAPAdminPasswordLine}" == "" ]; then
    mspOSMiniSAPAdminPassword="${mspOSMiniSAPAdminPasswordDefault}"
    mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPConfigurationOSAdminPassword}=\"${mspOSMiniSAPAdminPassword}\""
  fi
fi
mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPAdminPasswordLine//\"/\'}";  # Replace all " with '.
if [ "${mspOSMiniSAPAdminPassword}" == "" ]; then
  mspOSMiniSAPAdminPassword="${mspOSMiniSAPAdminPasswordLine#*=}"
  mspOSMiniSAPAdminPassword="${mspOSMiniSAPAdminPassword#\'}"; mspOSMiniSAPAdminPassword="${mspOSMiniSAPAdminPassword%\'}";
fi
if [ "${mspOSMiniSAPAdminPassword}" == "" ]; then
    mspOSMiniSAPAdminPassword="${mspOSMiniSAPAdminPasswordDefault}"
    mspOSMiniSAPAdminPasswordLine="${mspOSMiniSAPConfigurationOSAdminUserName}=\"${mspOSMiniSAPAdminPassword}\""
fi

# Later: Once you know the .desktop 'TryExec=?' touch file for execution to
#        detect existance of HANA stack install (not ABAP stack) then add
#        if/then/else here for ... mspOSMiniSAPAdminUserNameDefault="hxeadmn".
if [ "${mspOSMiniSAPAdminUserNameDefault}" == "" ]; then
  if [ -f "/sapmnt/NPL/exe/uc/linuxx86_64/startsap" ] ;then
    mspOSMiniSAPAdminUserNameDefault="npladm"
# elif [ -f "/sapmnt/NPL/exe/uc/linuxx86_64/startsap" ] ;then
#   mspOSMiniSAPAdminUserNameDefault="hxeadm"
  fi
fi

# Check for user name to be used.
mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPConfigurationOSAdminUserName}='${mspOSMiniSAPAdminUserName}'"
if [ "${mspOSMiniSAPAdminUserName}" == "" ]; then
  mspOSMiniSAPAdminUserNameLine="$(grep --no-messages -i -e "^${mspOSMiniSAPConfigurationOSAdminUserName}=.*" ${mspMiniSAPConfigurationLocalFQN})"
  if [ "${mspOSMiniSAPAdminUserNameLine}" == "" ]; then
    mspOSMiniSAPAdminUserName="${mspOSMiniSAPAdminUserNameDefault}"
    mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPConfigurationOSAdminUserName}=\"${mspOSMiniSAPAdminUserName}\""
  fi
fi
mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPAdminUserNameLine//\"/\'}";  # Replace all " with '.
if [ "${mspOSMiniSAPAdminUserName}" == "" ]; then
  mspOSMiniSAPAdminUserName="${mspOSMiniSAPAdminUserNameLine#*=}"
  mspOSMiniSAPAdminUserName="${mspOSMiniSAPAdminUserName#\'}"; mspOSMiniSAPAdminUserName="${mspOSMiniSAPAdminUserName%\'}";
fi
if [ "${mspOSMiniSAPAdminUserName}" == "" ]; then
    mspOSMiniSAPAdminUserName="${mspOSMiniSAPAdminUserNameDefault}"
    mspOSMiniSAPAdminUserNameLine="${mspOSMiniSAPConfigurationOSAdminUserName}=\"${mspOSMiniSAPAdminUserName}\""
fi

if [ "${mspDoVerbose}" != "0" ]; then
  echo "[MiniSAP+] Stop MiniSAP services healthcheck starting ..."
  echo "[MiniSAP+] Execution Directory ..... : ${mspScriptDirectory}"
  echo "[MiniSAP+] Execution Filename ...... : ${mspScriptFilename}"
  echo "[MiniSAP+] MiniSAP OS Admin User Name: ${mspOSMiniSAPAdminUserName}"
  echo "[MiniSAP+] MiniSAP OS Admin Password : ${mspOSMiniSAPAdminPassword}"
fi

if [ "${mspOSMiniSAPAdminUserName}" == "" ] ||
   [ "${mspOSMiniSAPAdminPassword}" == "" ]; then
  if [ "${mspDoSilent}" == "0" ]; then
    echo "[MiniSAP+] Stop MiniSAP services aborted."
    echo "[MiniSAP+]"
    echo "[MiniSAP+] Error - Unable to determine User Name and Password to use."
    echo "[MiniSAP+]"
    echo "[MiniSAP+] Option 1: Use '${mspScriptFilename} -u <username> -p <password>'"
    echo "[MiniSAP+]           to do this action one-time."
    echo "[MiniSAP+]"
    echo "[MiniSAP+] Option 2: Use 'minisap-plus.sh -u <username> -p <password>' to save"
    echo "[MiniSAP+]           them as the default user name and password, for later."
    echo "[MiniSAP+]      Tip: Now you can call just '${mspScriptFilename}' and it"
    echo "[MiniSAP+]           will use the saved user name and password, everytime."
    echo "[MiniSAP+]"
    echo "[MiniSAP+] Note: The <username> and <password> should be the guest operating"
    echo "[MiniSAP+]       systems user account that was created during the installation"
    echo "[MiniSAP+]       of the MiniSAP Developer Edition."
    echo "[MiniSAP+]       Like 'npladm' for the SAP ABAP stack technologies."
    echo "[MiniSAP+]       Like 'hxeadm' for the SAP HANA stack technologies."
    echo "[MiniSAP+]       ... and the password you entered during that installation."
    echo "[MiniSAP+]"
    echo "[MiniSAP+] Use '${mspScriptDirectory}${mspScriptFilename} --help' option for full details."
  fi
  mspScriptExitNow
  exit 2;  # 2 is 'Error - Unable to determine username and password to use.'.
fi

#-------------------------------------------------------------------------------
# Lets do it ... assuming use of Gnome desktop in OpenSUSE 15 (Leap).
# ... because this script is a companion for <https://www.installminisap.com/>.

if [ "${mspDoSilent}" == "0" ]; then
  echo "[MiniSAP+] Stopping MiniSAP services ..."
fi
echo -e "${mspOSMiniSAPAdminPassword}\n" | sudo -k -u ${mspOSMiniSAPAdminUserName} -S -i stopsap
if [ "${mspDoSilent}" == "0" ]; then
  echo "[MiniSAP+] Stopping MiniSAP services has finished."
fi

#-------------------------------------------------------------------------------
# Housekeeping and finish.

mspScriptExitNow
exit 0;  # 0 is 'Ok'.

################################################################################